@@ -18,94 +18,11 @@
 #
 
 set tix_demo_running 1
+set samples_dir [tixNSubFolder $demo_dir samples]
+set sample_filename {}
 
-# types: "d" directory "f" file
-# conditions: "i" image "c" command "v" variable
-set root {
-    {d "File Selectors"		file	}
-    {d "Hierachical ListBox"	hlist	}
-    {d "Tabular ListBox"	tlist	{c tixTList}}
-    {d "Grid Widget"		grid	{c tixGrid}}
-    {d "Manager Widgets"	manager	}
-    {d "Scrolled Widgets"	scroll	}
-    {d "Miscellaneous Widgets"	misc	}
-    {d "Image Types"		image	}
-}
+uplevel #0 source [list [tixNSubFolder $samples_dir AllSampl.tcl]]
 
-#
-#    These are only alpha codes ...
-#    {"Drag and Drop"		DragDrop.tcl}
-#
-
-set image {
-    {d "Compound Image"		cmpimg	}
-    {d "XPM Image"		xpm	{i pixmap}}
-}
-
-set cmpimg {
-    {f "In Buttons"		CmpImg.tcl	}
-    {f "In NoteBook"		CmpImg2.tcl	}
-    {f "Icons"			CmpImg3.tcl	}
-}
-
-set xpm {
-    {f "In Button"		Xpm.tcl		{i pixmap}}
-    {f "In Menu"		Xpm1.tcl	{i pixmap}}
-}
-
-set file {
-    {f DirList				DirList.tcl	}
-    {f DirTree				DirTree.tcl	}
-    {f ExFileSelectDialog		EFileDlg.tcl	}
-    {f FileSelectDialog			FileDlg.tcl	}
-    {f FileEntry			FileEnt.tcl	}
-}
-
-set hlist {
-    {f HList			HList1.tcl	}
-    {f CheckList		ChkList.tcl	{c tixCheckList}}
-    {f "ScrolledHList (1)"	SHList.tcl	}
-    {f "ScrolledHList (2)"	SHList2.tcl	}
-    {f Tree			Tree.tcl	}
-    {f "Tree (Dynamic)"		DynTree.tcl	{v win}}
-}
-
-set tlist {
-    {f "ScrolledTList (1)"	STList1.tcl	{c tixTList}}
-    {f "ScrolledTList (2)"	STList2.tcl	{c tixTList}}
-    {f "TList File Viewer"	STList3.tcl	{c tixTList}}
-}
-
-set grid {
-    {f "Simple Grid"		SGrid0.tcl	{c tixGrid}}
-    {f "ScrolledGrid"		SGrid1.tcl	{c tixGrid}}
-}
-
-set scroll {
-    {f ScrolledListBox		SListBox.tcl	}
-    {f ScrolledText		SText.tcl	}
-    {f ScrolledWindow		SWindow.tcl	}
-}
-
-set manager {
-    {f ListNoteBook		ListNBK.tcl	}
-    {f NoteBook			NoteBook.tcl	}
-    {f PanedWindow		PanedWin.tcl	}
-}
-
-set misc {
-    {f Balloon			Balloon.tcl	}
-    {f ButtonBox		BtnBox.tcl	}
-    {f ComboBox			ComboBox.tcl	}
-    {f Control			Control.tcl	}
-    {f LabelEntry		LabEntry.tcl	}
-    {f LabelFrame		LabFrame.tcl	}
-    {f Meter			Meter.tcl	{c tixMeter}}
-    {f OptionMenu		OptMenu.tcl	}
-    {f PopupMenu		PopMenu.tcl	}
-    {f Select			Select.tcl	}
-    {f StdButtonBox		StdBBox.tcl	}
-}
 
 proc MkSample {nb page} {
     global tixOption
@@ -196,94 +113,36 @@ proc MkSample {nb page} {
     set file   [tix getimage textfile]
     set folder [tix getimage openfold]
 
-    AddSamples $tree $hlist root "" $style $file $folder
-
-    global env sampleHlist
-    uplevel #0 set sampleHlist $hlist 
-    if [info exists env(TIX_DO_ALL_DEMOS)] {
-	# This is undocumented feature. Don't do it.
-	#
-	after 200 DoAll $hlist
-    }
+    ForAllSamples root "" \
+	[list AddSampleToHList $tree $hlist $style $file $folder]
 }
 
-
-proc DoAll {hlist {path ""}} {
-    catch {
-	set theSample [$hlist info data $path]
-	if {$theSample != {}} {
-	    set title [lindex $theSample 0]
-	    set prog  [lindex $theSample 1]
-
-	    RunProg $title $prog
-	   update
+# AddSampleToHList --
+#
+#	A callback from ForAllSamples. Add all the possible sample files
+#	into the Tree widget.
+#
+proc AddSampleToHList {tree hlist style file folder token type text dest} {
+    case $type {
+	d {
+	    return [$hlist addchild $token -itemtype imagetext -style $style \
+		-image $folder -text $text]
 	}
-    }
-
-    foreach p [$hlist info children $path] {
-	DoAll $hlist $p
-    }    
-}
-
-proc AddSamples {t h name ent style fileIm folderIm} {
-    global $name win
-
-    if {[tix platform] == "windows"} {
-	set win 1
-    }
-
-    foreach line [set $name] {
-	set type [lindex $line 0]
-	set text [lindex $line 1]
-	set dest [lindex $line 2]
-	set cond [lindex $line 3]
-
-	case [lindex $cond 0] {
-	    c {
-		set cmd [lindex $cond 1]
-		if {[info command $cmd] != $cmd} {
-		    if ![auto_load $cmd] {
-			continue
-		    }
-		}
-	    }
-	    i {
-		if {[lsearch [image types] [lindex $cond 1]] == -1} {
-		    continue
-		}
-	    }
-	    v {
-		set doit 1
-		foreach var [lrange $cond 1 end] {
-		    if [uplevel #0 info exists [list $var]] {
-			set doit 0
-			break
-		    }
-		}
-		if !$doit {
-		    continue
-		}
+	done {
+	    if {![tixStrEq $token ""]} {
+		$tree setmode $token close
+		$tree close $token
 	    }
 	}
-
-	if {$type == "d"} {
-	    set e [$h addchild $ent -itemtype imagetext -style $style \
-		-image $folderIm -text $text]
-	    
-	    AddSamples $t $h $dest $e $style $fileIm $folderIm
-	    $t setmode $e close
-	    $t close $e
-	} else {
-	    set e [$h addchild $ent -itemtype imagetext \
-		-image $fileIm -text $text -data [list $text $dest]]
+	f {
+	    return [$hlist addchild $token -itemtype imagetext \
+		-image $file -text $text -data [list $text $dest]]
 	}
     }
 }
 
-set sample_filename {}
-
 proc Sample:Action {w slb action args} {
-    global samples demo_dir demo
+    global samples demo_dir demo samples_dir
 
     set hlist [$slb subwidget hlist]
     set ent [$hlist info anchor]
@@ -307,8 +166,6 @@ proc Sample:Action {w slb action args} {
     set title [lindex $theSample 0]
     set prog  [lindex $theSample 1]
 
-    set samples_dir [tixNSubFolder $demo_dir samples]
-
     case $action {
 	"run" {
 	    RunProg $title $prog
@@ -334,9 +191,8 @@ proc Sample:Action {w slb action args} {
 }
 
 proc RunProg {title prog} {
-    global samples demo_dir demo
+    global samples demo_dir demo samples_dir
 
-    set samples_dir [tixNSubFolder $demo_dir samples]
     set w .[lindex [split $prog .] 0]
     set w [string tolower $w]
 
@@ -389,8 +245,10 @@ proc LoadFile {filename} {
 
 proc ReadFileWhenIdle {text} {
     global sample_filename
-    
-    ReadFile $text $sample_filename
+
+    if ![file isdir $sample_filename] {
+	ReadFile $text $sample_filename
+    }
 }
 
 proc ReadFile {text filename} {
