@@ -1,6 +1,3 @@
-
-/*	$Id: tix.h,v 1.4.2.4 2002/12/02 04:02:51 idiscovery Exp $	*/
-
 /*
  * tix.h --
  *
@@ -20,18 +17,12 @@
 #define _TIX_H_
 
 #ifndef TIX_VERSION
-#define TIX_VERSION	"8.1"
+#define TIX_VERSION	"4.1"
 #endif
-#define TIX_PATCHLEVEL	"8.1.4"
+#define TIX_PATCHLEVEL	"4.1.0"
 #define TIX_PATCH_LEVEL TIX_PATCHLEVEL
 
-#define TIX_RELEASE     "8.1.4"
-
-/* We can use USE_NON_CONST or USE_COMPAT_CONST to see the compiler warnings */
-# define USE_NON_CONST
-/* We can start flagging CONST errors with tk8.4 by defining this instead
-   see http://mini.net/tcl/3669 */
-/* #define USE_COMPAT_CONST */
+#define TIX_RELEASE     "4.1.0.001"
 
 #ifndef _TK
 #include <tk.h>
@@ -41,53 +32,12 @@
 #include <tcl.h>
 #endif
 
-/* Move forward to tcl 8.4 but maintain backward compatability */
-#ifndef CONST84
-#   define CONST84
-#endif
-
 #if defined(__WIN32__) || defined(_WIN32) || defined (__BORLAND) || defined(_Windows)
 #ifndef _WINDOWS
 #define _WINDOWS
 #endif
 #endif
 
-/*
- * When building Tix itself, BUILD_tix should be defined by the makefile
- * so that all EXTERN declarations get DLLEXPORT; when building apps
- * using Tix, BUILD_tix should NOT be defined so that all EXTERN
- * declarations get DLLIMPORT as defined in tcl.h
- *
- * NOTE: This ifdef MUST appear after the include of tcl.h and tk.h
- * because the EXTERN declarations in those files need DLLIMPORT.
- */
-/*
- * These macros are used to control whether functions are being declared for
- * import or export.  If a function is being declared while it is being built
- * to be included in a shared library, then it should have the DLLEXPORT
- * storage class.  If is being declared for use by a module that is going to
- * link against the shared library, then it should have the DLLIMPORT storage
- * class.  If the symbol is beind declared for a static build or for use from a
- * stub library, then the storage class should be empty.
- *
- * The convention is that a macro called BUILD_xxxx, where xxxx is the
- * name of a library we are building, is set on the compile line for sources
- * that are to be placed in the library.  When this macro is set, the
- * storage class will be set to DLLEXPORT.  At the end of the header file, the
- * storage class will be reset to DLLIMPORt.
- */
-
-#undef TCL_STORAGE_CLASS
-#ifdef BUILD_tix
-# define TCL_STORAGE_CLASS DLLEXPORT
-#else
-# ifdef USE_TCL_STUBS
-#  define TCL_STORAGE_CLASS
-# else
-#  define TCL_STORAGE_CLASS DLLIMPORT
-# endif
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -109,10 +59,6 @@ extern "C" {
 #  endif
 #endif /* TK_MAJOR_VERSION ... */
 
-#if (TK_MAJOR_VERSION >= 8)
-#  define TK_8_0_OR_LATER
-#endif
-
 #ifdef TK_4_1_OR_LATER
     /* TK 4.1 or later */
 #  define Tix_FreeProc Tcl_FreeProc
@@ -426,8 +372,6 @@ EXTERN void 		Tix_SetArgv _ANSI_ARGS_((Tcl_Interp *interp,
 			    int argc, char **argv));
 EXTERN void		Tix_SetRcFileName _ANSI_ARGS_((
 			    Tcl_Interp * interp,  char * rcFileName));
-EXTERN char *		TixGetStringFromObj _ANSI_ARGS_((
-			    char *objPtr,int *lengthPtr));
 
 
 /*
@@ -469,6 +413,11 @@ extern TIX_DECLARE_CMD(Tix_UnmapWindowCmd);
 extern TIX_DECLARE_CMD(Tix_MwmCmd);
 extern TIX_DECLARE_CMD(Tix_CreateWidgetCmd);
 
+#ifdef TCL_7_5_OR_LATER
+extern TIX_DECLARE_CMD(Tix_ConsoleInitCmd);
+#endif
+
+
 #define SET_RECORD(interp, record, var, value) \
 	Tcl_SetVar2(interp, record, var, value, TCL_GLOBAL_ONLY)
 
@@ -480,12 +429,26 @@ extern TIX_DECLARE_CMD(Tix_CreateWidgetCmd);
 
 /*----------------------------------------------------------------------
  * Compatibility section
- *----------------------------------------------------------------------
- */
+ *----------------------------------------------------------------------	*/
+
+#ifdef NO_STRDUP
+#define strdup tixStrDup
+
+EXTERN char * 		tixStrDup _ANSI_ARGS_((
+			    CONST char * s));
+#endif
 
 #ifdef _WINDOWS
-/* This is the way win/tkWinPort.h in tk8.0.5 defines it */
-#define strcasecmp stricmp
+#ifndef NO_STRCASECMP
+#define NO_STRCASECMP 1
+#endif
+#endif
+
+#if defined(NO_STRCASECMP)
+#define strcasecmp tixStrCaseCmp
+
+extern int		tixStrCaseCmp _ANSI_ARGS_((CONST char *s1,
+			    CONST char *s2));
 #endif
 
 
@@ -495,4 +458,3 @@ extern TIX_DECLARE_CMD(Tix_CreateWidgetCmd);
 
 
 #endif /* _TIX_H_ */
-
