@@ -10,12 +10,12 @@
  *
  */
 
-#include <tkInt.h>
-#include <tkPort.h>
-#include <tix.h>
+#include <tixPort.h>
+#include <tixInt.h>
 #include <tixDef.h>
 
-/* ToDo: 
+/*
+ * ToDo: 
  *	- lineconfig and itemconfig command
  */
 
@@ -46,7 +46,7 @@ typedef struct CmpMaster {
     int relief;			/* Indicates whether window as a whole is
 				 * raised, sunken, or flat. */
 
-    XFontStruct *fontPtr;	/* Information about text font.*/
+    TixFont font;		/* Information about text font.*/
     XColor *foreground;		/* Color for drawing text and bitmaps */
     GC gc;			/* default GC for drawing text. */
 
@@ -119,7 +119,7 @@ typedef struct CmpTextItem {
     int underline;		/* Index of character to underline.  < 0 means
 				 * don't underline anything. */
     XColor *foreground;
-    XFontStruct *fontPtr;	/* Information about text font, or NULL. */
+    TixFont font;		/* Information about text font, or NULL. */
     GC gc;			/* GC for drawing the bitmap. */
 } CmpTextItem;
 
@@ -180,7 +180,7 @@ static Tk_ConfigSpec configSpecs[] = {
        (char *) NULL, 0, 0},
 
     {TK_CONFIG_FONT, "-font", "font", "Font",
-       DEF_CMPIMAGE_FONT, Tk_Offset(CmpMaster, fontPtr), 0},
+       DEF_CMPIMAGE_FONT, Tk_Offset(CmpMaster, font), 0},
 
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
        DEF_CMPIMAGE_FG_COLOR, Tk_Offset(CmpMaster, foreground),
@@ -281,7 +281,7 @@ static Tk_ConfigSpec textConfigSpecs[] = {
        (char *) NULL, 0, 0},
 
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
-       "", Tk_Offset(CmpTextItem, fontPtr), TK_CONFIG_NULL_OK},
+       "", Tk_Offset(CmpTextItem, font), TK_CONFIG_NULL_OK},
 
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
        "", Tk_Offset(CmpTextItem, foreground),
@@ -334,6 +334,8 @@ CmpTextItem * 		AddNewText _ANSI_ARGS_((CmpMaster *masterPtr,
 			    int argc, char **argv));
 CmpLine* 		AddNewLine _ANSI_ARGS_((CmpMaster *masterPtr,
 			    int argc, char **argv));
+static void		CalculateMasterSize _ANSI_ARGS_((
+			    ClientData clientData));
 static void		ChangeImageWhenIdle _ANSI_ARGS_((
 			    CmpMaster *masterPtr));
 static void		ImageProc _ANSI_ARGS_((ClientData clientData,
@@ -393,7 +395,7 @@ ImgCmpCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     masterPtr->borderWidth = 0;
     masterPtr->background = NULL;
     masterPtr->relief = 0;
-    masterPtr->fontPtr = NULL;
+    masterPtr->font = NULL;
     masterPtr->foreground = NULL;
     masterPtr->gc = None;
     masterPtr->showBackground = 0;
@@ -474,7 +476,7 @@ ImgCmpConfigureMaster(masterPtr, argc, argv, flags)
      */
     gcValues.foreground = masterPtr->foreground->pixel;
     gcValues.background = Tk_3DBorderColor(masterPtr->background)->pixel;
-    gcValues.font = masterPtr->fontPtr->fid;
+    gcValues.font = TixFontId(masterPtr->font);
     gcValues.graphics_exposures = False;
     newGC = Tk_GetGC(masterPtr->tkwin,
 	 GCBackground|GCForeground|GCFont|GCGraphicsExposures,
@@ -864,7 +866,7 @@ AddNewText(masterPtr, line, argc, argv)
     p.text->wrapLength = 0;
 
     p.text->foreground = NULL;
-    p.text->fontPtr = NULL;
+    p.text->font = NULL;
     p.text->gc = None;
 
     if (Tk_ConfigureWidget(masterPtr->interp, masterPtr->tkwin,
@@ -880,10 +882,10 @@ AddNewText(masterPtr, line, argc, argv)
     } else {
 	gcValues.foreground = masterPtr->foreground->pixel;
     }
-    if (p.text->fontPtr) {
-	gcValues.font = p.text->fontPtr->fid;
+    if (p.text->font) {
+	gcValues.font = TixFontId(p.text->font);
     } else {
-	gcValues.font = masterPtr->fontPtr->fid;
+	gcValues.font = TixFontId(masterPtr->font);
     }
     gcValues.graphics_exposures = False;
     p.text->gc = Tk_GetGC(masterPtr->tkwin,
@@ -970,20 +972,20 @@ CalculateMasterSize(clientData)
 
 	      case TYPE_TEXT:
 		{
-		    XFontStruct *fontPtr;
+		    TixFont font;
 
 		    if (p.text->text == NULL) {
 			break;
 		    }
 		  
-		    if (p.text->fontPtr) {
-			fontPtr = p.text->fontPtr;
+		    if (p.text->font) {
+			font = p.text->font;
 		    } else {
-			fontPtr = masterPtr->fontPtr;
+			font = masterPtr->font;
 		    }
 		    
 		    p.text->numChars = strlen(p.text->text);
-		    TkComputeTextGeometry(fontPtr, p.text->text,
+		    TixComputeTextGeometry(font, p.text->text,
 			p.text->numChars,
 			p.text->wrapLength,
 			&p.text->width, &p.text->height);
@@ -1136,20 +1138,20 @@ ImgCmpDisplay(clientData, display, drawable, imageX, imageY, width,
 
 	      case TYPE_TEXT:
 		{
-		    XFontStruct *fontPtr;
+		    TixFont font;
 
 		    if (p.text->text == NULL) {
 			break;
 		    }
 		  
-		    if (p.text->fontPtr) {
-			fontPtr = p.text->fontPtr;
+		    if (p.text->font) {
+			font = p.text->font;
 		    } else {
-			fontPtr = masterPtr->fontPtr;
+			font = masterPtr->font;
 		    }
 
-		    TkDisplayText(Tk_Display(masterPtr->tkwin), drawable,
-			fontPtr, p.text->text, p.text->numChars,
+		    TixDisplayText(Tk_Display(masterPtr->tkwin), drawable,
+			font, p.text->text, p.text->numChars,
 			dx, dy+extraY,	    
 			p.text->width - 2*p.item->padX,
 			p.text->justify,
@@ -1329,8 +1331,9 @@ ImgCmpDelete(masterData)
     }
     masterPtr->tkMaster = NULL;
     if (masterPtr->imageCmd != NULL) {
-	Tcl_DeleteCommand(masterPtr->interp,
-	    Tcl_GetCommandName(masterPtr->interp, masterPtr->imageCmd));
+	char * cmd = Tcl_GetCommandName(masterPtr->interp,masterPtr->imageCmd);
+	masterPtr->imageCmd = NULL;
+	Tcl_DeleteCommand(masterPtr->interp, cmd);
     }
     if (masterPtr->gc != None) {
 	Tk_FreeGC(Tk_Display(masterPtr->tkwin), masterPtr->gc);
@@ -1434,7 +1437,14 @@ CmpEventProc(clientData, eventPtr)
     ClientData clientData;	/* Information about window. */
     XEvent *eventPtr;		/* Information about event. */
 {
+    CmpMaster *masterPtr = (CmpMaster *)clientData;
+    char * cmd;
+
     if (eventPtr->type == DestroyNotify) {
-	ImgCmpCmdDeletedProc(clientData);
+	if (masterPtr->imageCmd != NULL) {
+	    cmd = Tcl_GetCommandName(masterPtr->interp,masterPtr->imageCmd);
+	    masterPtr->imageCmd = NULL;
+	    Tcl_DeleteCommand(masterPtr->interp, cmd);
+	}
     }
 }
