@@ -1,140 +1,21 @@
-/**
+/*
  * tixImgXpm.c --
  *
  *	This file implements images of type "pixmap" for Tix.
- * ______________________________________________________________________
  *
- * Copyright statement for tixImgXpm.c
- *	Copyright 1996, Expert Interface Technologies
+ * Copyright (c) 1996, Expert Interface Technologies
  *
- * The following terms apply only to this file and no other parts of the
- * Tix library.
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- *	Permission is hereby granted, without written agreement and
- *	without license or royalty fees, to use, copy, modify, and
- *	distribute this file, for any purpose, provided that existing
- *	copyright notices are retained in all copies and that this
- *	notice is included verbatim in any distributions.
- *
- *	DISCLAIMER OF ALL WARRANTIES
- *
- *	IN NO EVENT SHALL THE AUTHOR OF THIS SOFTWARE BE LIABLE TO ANY
- *	PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
- *	CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE
- *	AND ITS DOCUMENTATION, EVEN IF THE AUTHOR OF THIS SOFTWARE HAS
- *	BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *	THE AUTHOR OF THIS SOFTWARE SPECIFICALLY DISCLAIMS ANY
- *	WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- *	PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
- *	BASIS, AND THE AUTHOR OF THIS SOFTWARE HAS NO OBLIGATION TO
- *	PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
- *	MODIFICATIONS.
- * ___________________________________________________________________
- *
- * This file is adapted from the Tk 4.0 source file tkImgBmap.c
- * Original tkImgBmap.c copyright information:
- *
- *	Copyright (c) 1994 The Regents of the University of California.
- *	Copyright (c) 1994-1995 Sun Microsystems, Inc.
- *
- *	See the file "license.terms.tcltk" for information on usage
- *	and redistribution of the original tkImgBmap.c file, and for a
- *	DISCLAIMER OF ALL WARRANTIES from the authors of tkImgBmap.c.
- */
-#include "tkInt.h"
-#include "tkPort.h"
-#include <tix.h>
-
-/* constants used only in this file */
-
-#define XPM_MONO		1
-#define XPM_GRAY_4		2
-#define XPM_GRAY		3
-#define XPM_COLOR		4
-#define XPM_SYMBOLIC		5
-#define XPM_UNKNOWN		6
-
-/*
- * The following data structure represents the master for a pixmap
- * image:
  */
 
-typedef struct PixmapMaster {
-    Tk_ImageMaster tkMaster;	/* Tk's token for image master.  NULL means
-				 * the image is being deleted. */
-    Tcl_Interp *interp;		/* Interpreter for application that is
-				 * using image. */
-    Tcl_Command imageCmd;	/* Token for image command (used to delete
-				 * it when the image goes away).  NULL means
-				 * the image command has already been
-				 * deleted. */
-    char *fileString;		/* Value of -file option (malloc'ed).
-				 * valid only if the -file option is specified
-				 */
-    char *dataString;		/* Value of -data option (malloc'ed).
-				 * valid only if the -data option is specified
-				 */
-				/* First in list of all instances associated
-				 * with this master. */
-    Tk_Uid id;			/* ID's for XPM data already compiled
-				 * into the tixwish binary */
-    int size[2];		/* width and height */
-    int ncolors;		/* number of colors */
-    int cpp;			/* characters per pixel */
-    char ** data;		/* The data that defines this pixmap 
-				 * image (array of strings). It is
-				 * converted into an X Pixmap when this
-				 * image is instanciated
-				 */
-    int isDataAlloced;		/* False iff the data is got from
-				 * the -id switch */
-    struct PixmapInstance *instancePtr;
-} PixmapMaster;
-
-/* Make this more portable */
-
-typedef struct ColorStruct {
-    char c;			/* This is used if CPP is one */
-    char * cstring;		/* This is used if CPP is bigger than one */
-    XColor * colorPtr;
-} ColorStruct;
+#include <tixPort.h>
+#include <tixInt.h>
+#include <tixImgXpm.h>
 
 /*
- * The following data structure represents all of the instances of an
- * image that lie within a particular window:
- *
- * %% ToDo
- * Currently one instance is created for each window that uses this pixmap.
- * This is usually OK because pixmaps are usually not shared or only shared by
- * a small number of windows. To improve resource allocation, we can
- * create an instance for each (Display x Visual x Depth) combo. This will
- * usually reduce the number of instances to one.
- */
-typedef struct PixmapInstance {
-    int refCount;		/* Number of instances that share this
-				 * data structure. */
-    PixmapMaster *masterPtr;	/* Pointer to master for image. */
-    Tk_Window tkwin;		/* Window in which the instances will be
-				 * displayed. */
-    Pixmap pixmap;		/* The pixmap to display. */
-    Pixmap mask;		/* Mask: only display pixmap pixels where
-				 * there are 1's here. */
-    GC gc;			/* Graphics context for displaying pixmap.
-				 * None means there was an error while
-				 * setting up the instance, so it cannot
-				 * be displayed. */
-    struct PixmapInstance *nextPtr;
-				/* Next in list of all instance structures
-				 * associated with masterPtr (NULL means
-				 * end of list).
-				 */
-    ColorStruct * colors;
-} PixmapInstance;
-
-/*
- * The type record for pixmap images:
+ * Prototypes for procedures used only locally in this file:
  */
 
 static int		ImgXpmCreate _ANSI_ARGS_((Tcl_Interp *interp,
@@ -150,35 +31,6 @@ static void		ImgXpmDisplay _ANSI_ARGS_((ClientData clientData,
 static void		ImgXpmFree _ANSI_ARGS_((ClientData clientData,
 			    Display *display));
 static void		ImgXpmDelete _ANSI_ARGS_((ClientData clientData));
-
-Tk_ImageType tixPixmapImageType = {
-    "pixmap",			/* name */
-    ImgXpmCreate,		/* createProc */
-    ImgXpmGet,			/* getProc */
-    ImgXpmDisplay,		/* displayProc */
-    ImgXpmFree,			/* freeProc */
-    ImgXpmDelete,		/* deleteProc */
-    (Tk_ImageType *) NULL	/* nextPtr */
-};
-
-/*
- * Information used for parsing configuration specs:
- */
-
-static Tk_ConfigSpec configSpecs[] = {
-    {TK_CONFIG_STRING, "-data", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PixmapMaster, dataString), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_STRING, "-file", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PixmapMaster, fileString), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_UID, "-id", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PixmapMaster, id), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
-	(char *) NULL, 0, 0}
-};
-
-/*
- * Prototypes for procedures used only locally in this file:
- */
 static int		ImgXpmCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		ImgXpmCmdDeletedProc _ANSI_ARGS_((
@@ -196,12 +48,44 @@ static char ** 		ImgXpmGetDataFromId _ANSI_ARGS_((Tcl_Interp * interp,
 			    char * id));
 static char ** 		ImgXpmGetDataFromString _ANSI_ARGS_((Tcl_Interp*interp,
 			    char * string, int * numLines_return));
-static int 		ImgXpmGetPixmapFromData _ANSI_ARGS_((
+static void 		ImgXpmGetPixmapFromData _ANSI_ARGS_((
 			    Tcl_Interp * interp,
 			    PixmapMaster *masterPtr,
 			    PixmapInstance *instancePtr));
+static char *		GetType _ANSI_ARGS_((char * colorDefn,
+			    int  * type_ret));
+static char *		GetColor _ANSI_ARGS_((char * colorDefn,
+			    char * colorName, int * type_ret));
+
+/*
+ * Information used for parsing configuration specs:
+ */
+
+static Tk_ConfigSpec configSpecs[] = {
+    {TK_CONFIG_STRING, "-data", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(PixmapMaster, dataString), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_STRING, "-file", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(PixmapMaster, fileString), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_UID, "-id", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(PixmapMaster, id), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
+	(char *) NULL, 0, 0}
+};
+
+Tk_ImageType tixPixmapImageType = {
+    "pixmap",			/* name */
+    ImgXpmCreate,		/* createProc */
+    ImgXpmGet,			/* getProc */
+    ImgXpmDisplay,		/* displayProc */
+    ImgXpmFree,			/* freeProc */
+    ImgXpmDelete,		/* deleteProc */
+    (Tk_ImageType *) NULL	/* nextPtr */
+};
+
+/*
+ * Local data, used only in this file
+ */
 
-/* Local data, used only in this file */
 static Tcl_HashTable xpmTable;
 static int xpmTableInited = 0;
 
@@ -334,11 +218,16 @@ ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
 	Tk_ImageChanged(masterPtr->tkMaster, 0, 0, 0, 0, 0, 0);
     }
 
-  done:
     return TCL_OK;
 
   error:
     /* Restore it to the original (possible valid) mode */
+    if (masterPtr->dataString && masterPtr->dataString != oldData) {
+	ckfree(masterPtr->dataString);
+    }
+    if (masterPtr->fileString && masterPtr->fileString != oldFile) {
+	ckfree(masterPtr->fileString);
+    }
     masterPtr->dataString = oldData;
     masterPtr->fileString = oldFile;
     masterPtr->id = oldId;
@@ -368,7 +257,7 @@ ImgXpmGetData(interp, masterPtr)
     Tcl_Interp *interp;			/* For reporting errors. */
     PixmapMaster *masterPtr;
 {
-    char ** data;
+    char ** data = NULL;
     int  isAllocated;			/* do we need to free "data"? */
     int listArgc;
     char ** listArgv = NULL;
@@ -376,6 +265,7 @@ ImgXpmGetData(interp, masterPtr)
     int size[2];
     int cpp;
     int ncolors;
+    int code = TCL_OK;
 
     if (masterPtr->id != NULL) {
 	data = ImgXpmGetDataFromId(interp, masterPtr->id);
@@ -395,30 +285,30 @@ ImgXpmGetData(interp, masterPtr)
     }
 
     if (data == NULL) {
+	/* nothing has been allocated yet. Don't need to goto done */
 	return TCL_ERROR;
     }
 
     /* Parse the first line of the data and get info about this pixmap */
     if (Tcl_SplitList(interp, data[0], &listArgc, &listArgv) != TCL_OK) {
-	goto error;
+	code = TCL_ERROR; goto done;
     }
 
-    if (listArgc < 4) {
-	Tcl_AppendResult(interp, "File format error", NULL);
-	goto error;
+    if (listArgc < 4) {	/* file format error */
+	code = TCL_ERROR; goto done;
     }
 
     if (Tcl_GetInt(interp, listArgv[0], &size[0]) != TCL_OK) {
-	goto error;
+	code = TCL_ERROR; goto done;
     }
     if (Tcl_GetInt(interp, listArgv[1], &size[1]) != TCL_OK) {
-	goto error;
+	code = TCL_ERROR; goto done;
     }
     if (Tcl_GetInt(interp, listArgv[2], &ncolors) != TCL_OK) {
-	goto error;
+	code = TCL_ERROR; goto done;
     }
     if (Tcl_GetInt(interp, listArgv[3], &cpp) != TCL_OK) {
-	goto error;
+	code = TCL_ERROR; goto done;
     }
 
     if (isAllocated) {
@@ -426,35 +316,35 @@ ImgXpmGetData(interp, masterPtr)
 	    /* the number of lines read from the file/data
 	     * is not the same as specified in the data
 	     */
-	    goto error;
+	    code = TCL_ERROR; goto done;
 	}
     }
 
   done:
-    if (masterPtr->isDataAlloced && masterPtr->data) {
-	ckfree((char*)masterPtr->data);
-    }
-    masterPtr->isDataAlloced = isAllocated;
-    masterPtr->data = data;
-    masterPtr->size[0] = size[0];
-    masterPtr->size[1] = size[1];
-    masterPtr->cpp = cpp;
-    masterPtr->ncolors = ncolors;
-    
-    return TCL_OK;
+    if (code == TCL_OK) {
+	if (masterPtr->isDataAlloced && masterPtr->data) {
+	    ckfree((char*)masterPtr->data);
+	}
+	masterPtr->isDataAlloced = isAllocated;
+	masterPtr->data = data;
+	masterPtr->size[0] = size[0];
+	masterPtr->size[1] = size[1];
+	masterPtr->cpp = cpp;
+	masterPtr->ncolors = ncolors;
+    } else {
+	if (isAllocated && data) {
+	    ckfree((char*)data);
+	}
 
-  error:
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp, "File format error", NULL);
-
-    if (isAllocated && data) {
-	ckfree((char*)data);
+	Tcl_ResetResult(interp);
+	Tcl_AppendResult(interp, "File format error", NULL);
     }
+
     if (listArgv) {
 	ckfree((char*)listArgv);
     }
 		   
-    return TCL_ERROR;
+    return code;
 }
 
 static char ** ImgXpmGetDataFromId(interp, id)
@@ -650,7 +540,6 @@ static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
     }
     cmdBuffer[size] = 0;
 
-  done:
     data = ImgXpmGetDataFromString(interp, cmdBuffer, numLines_return);
     ckfree(cmdBuffer);
     Tcl_DStringFree(&buffer);
@@ -665,7 +554,8 @@ static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
 }
 
 
-static char * GetType(colorDefn, type_ret)
+static char *
+GetType(colorDefn, type_ret)
     char * colorDefn;
     int  * type_ret;
 {
@@ -711,13 +601,16 @@ static char * GetType(colorDefn, type_ret)
     return p;
 }
 
-/* colorName is guaranteed to be big enough */
-static char * GetColor(colorDefn, colorName, type_ret)
+/*
+ * colorName is guaranteed to be big enough
+ */
+static char *
+GetColor(colorDefn, colorName, type_ret)
     char * colorDefn;
     char * colorName;		/* if found, name is copied to this array */
     int  * type_ret;
 {
-    int n, type;
+    int type;
     char * p;
 
     if (!colorDefn) {
@@ -777,27 +670,17 @@ static char * GetColor(colorDefn, colorName, type_ret)
  *	Creates a pixmap for an image instance.
  *----------------------------------------------------------------------
  */
-static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
+static void
+ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
     Tcl_Interp * interp;
     PixmapMaster *masterPtr;
     PixmapInstance *instancePtr;
 {
     XImage * image = NULL, * mask = NULL;
-    int pad, depth, i, j, k, n, lOffset, isTransp = 0, isMono;
+    int depth, i, j, k, lOffset, isTransp = 0, isMono;
     ColorStruct * colors;
-    GC gc;
-    Display *display = Tk_Display(instancePtr->tkwin);
 
     depth = Tk_Depth(instancePtr->tkwin);
-    if (depth > 16) {
-	pad = 32;
-    }
-    else if (depth > 8) {
-	pad = 16;
-    }
-    else {
-	pad = 8;
-    }
 
     switch ((Tk_Visual(instancePtr->tkwin))->class) {
       case StaticGray:
@@ -808,22 +691,7 @@ static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 	isMono = 0;
     }
 
-    /*
-     * Create the XImage structures to store the temporary image
-     */
-    image = XCreateImage(display,
-	Tk_Visual(instancePtr->tkwin),
-	depth, ZPixmap, 0, 0,
-	masterPtr->size[0], masterPtr->size[1], pad, 0);
-    image->data =
-      (char *)ckalloc(image->bytes_per_line * masterPtr->size[1]);
-
-    mask  = XCreateImage(display,
-	Tk_Visual(instancePtr->tkwin),
-	1, ZPixmap, 0, 0,
-	masterPtr->size[0], masterPtr->size[1], pad, 0);
-    mask->data =
-      (char *)ckalloc(mask->bytes_per_line  * masterPtr->size[1]);
+    TixpXpmAllocTmpBuffer(masterPtr, instancePtr, &image, &mask);
 
     /*
      * Parse the colors
@@ -831,7 +699,9 @@ static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
     lOffset = 1;
     colors = (ColorStruct*)ckalloc(sizeof(ColorStruct)*masterPtr->ncolors);
 
-    /* Initialize the color structures */
+    /*
+     * Initialize the color structures
+     */
     for (i=0; i<masterPtr->ncolors; i++) {
 	colors[i].colorPtr = NULL;
 	if (masterPtr->cpp == 1) {
@@ -940,13 +810,8 @@ static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 	    if (masterPtr->cpp == 1) {
 		for (k=0; k<masterPtr->ncolors; k++) {
 		    if (*p == colors[k].c) {
-			if (colors[k].colorPtr != NULL) {
-			    XPutPixel(image, j, i, colors[k].colorPtr->pixel);
-			    XPutPixel(mask,  j, i, 1);
-			} else {
-			    XPutPixel(mask,  j, i, 0);
-			    isTransp = 1;
-			}
+			TixpXpmSetPixel(instancePtr, image, mask, j, i,
+			        colors[k].colorPtr, &isTransp);
 			break;
 		    }
 		}
@@ -957,13 +822,8 @@ static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 		for (k=0; k<masterPtr->ncolors; k++) {
 		    if (strncmp(p, colors[k].cstring, 
 			    (size_t)masterPtr->cpp) == 0) {
-			if (colors[k].colorPtr != NULL) {
-			    XPutPixel(image, j, i, colors[k].colorPtr->pixel);
-			    XPutPixel(mask,  j, i, 1);
-			} else {
-			    XPutPixel(mask,  j, i, 0);
-			    isTransp = 1;
-			}
+			TixpXpmSetPixel(instancePtr, image, mask, j, i,
+			        colors[k].colorPtr, &isTransp);
 			break;
 		    }
 		}
@@ -974,48 +834,10 @@ static int ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 	}
     }
 
-    /*
-     * Create the pixmap(s) from the XImage structure. The mask is created
-     * only if needed (i.e., there is at least one transparent pixel)
-     */
     instancePtr->colors = colors;
 
-    /* main image */
-    instancePtr->pixmap = Tk_GetPixmap(display,
-	Tk_WindowId(instancePtr->tkwin),
-	masterPtr->size[0], masterPtr->size[1], depth);
-
-    gc = Tk_GetGC(instancePtr->tkwin, 0, NULL);
-
-    TkPutImage(NULL, 0, display, instancePtr->pixmap,
-	gc, image, 0, 0, 0, 0, masterPtr->size[0], masterPtr->size[1]);
-
-    Tk_FreeGC(display, gc);
-
-    /* mask, if necessary */
-    if (isTransp) {
-	instancePtr->mask = Tk_GetPixmap(display,
-	    Tk_WindowId(instancePtr->tkwin),
-	    masterPtr->size[0], masterPtr->size[1], 1);
-	gc = XCreateGC(display, instancePtr->mask, 0, NULL);
-	TkPutImage(NULL, 0, display, instancePtr->mask,
-	    gc, mask,  0, 0, 0, 0, masterPtr->size[0], masterPtr->size[1]);
-	XFreeGC(display, gc);
-    } else {
-	instancePtr->mask = None;
-    }
-
-    /* Done */
-    if (image) {
-	ckfree((char*)image->data);
-	image->data = NULL;
-	XDestroyImage(image);
-    }
-    if (mask) {
-	ckfree((char*)mask->data);
-	mask->data = NULL;
-	XDestroyImage(mask);
-    }
+    TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp);
+    TixpXpmFreeTmpBuffer(masterPtr, instancePtr, image, mask);
 }
 
 /*
@@ -1042,16 +864,11 @@ ImgXpmConfigureInstance(instancePtr)
     PixmapInstance *instancePtr;	/* Instance to reconfigure. */
 {
     PixmapMaster *masterPtr = instancePtr->masterPtr;
-    XGCValues gcValues;
-    GC gc;
-    unsigned int gcMask;
 
     if (instancePtr->pixmap != None) {
-	XFreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->pixmap);
-    }
-    if (instancePtr->mask != None) {
-	XFreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->mask);
+	Tk_FreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->pixmap);
     }
+    TixpXpmFreeInstanceData(instancePtr, 0);
 
     if (instancePtr->colors != NULL) {
 	int i;
@@ -1070,30 +887,13 @@ ImgXpmConfigureInstance(instancePtr)
 	Tk_MakeWindowExist(instancePtr->tkwin);
     }
 
-    /* Assumption: masterPtr->data is always non NULL (enfored by
+    /*
+     * Assumption: masterPtr->data is always non NULL (enfored by
      * ImgXpmConfigureMaster()). Also, the data must be in a valid
      * format (partially enforced by ImgXpmConfigureMaster(), see comments
      * inside that function).
      */
     ImgXpmGetPixmapFromData(masterPtr->interp, masterPtr, instancePtr);
-
-    /* Allocate a GC for drawing this instance (mask is not used if there
-     * is no transparent pixels inside the image).*/
-    if (instancePtr->mask != None) {
-	gcMask = GCGraphicsExposures|GCClipMask;
-    } else {
-	gcMask = GCGraphicsExposures;
-    }
-    gcValues.graphics_exposures = False;
-    gcValues.clip_mask = instancePtr->mask;
-    
-    gc = Tk_GetGC(instancePtr->tkwin, gcMask, &gcValues);
-
-    if (instancePtr->gc != None) {
-	Tk_FreeGC(Tk_Display(instancePtr->tkwin), instancePtr->gc);
-    }
-    instancePtr->gc = gc;
-    return;
 }
 
 /*
@@ -1133,6 +933,7 @@ ImgXpmCmd(clientData, interp, argc, argv)
     }
     c = argv[1][0];
     length = strlen(argv[1]);
+
     if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
 	    && (length >= 2)) {
 	if (argc != 3) {
@@ -1156,13 +957,26 @@ ImgXpmCmd(clientData, interp, argc, argv)
 		    TK_CONFIG_ARGV_ONLY);
 	}
 	return code;
+    } else if ((c == 'r') && (strncmp(argv[1], "refcount", length) == 0)) {
+	/*
+	 * The "refcount" command is for debugging only
+	 */
+	PixmapInstance *instancePtr;
+	int count = 0;
+	char buff[30];
+
+	for (instancePtr=masterPtr->instancePtr; instancePtr;
+	     instancePtr = instancePtr->nextPtr) {
+	    count += instancePtr->refCount;
+	}
+	sprintf(buff, "%d", count);
+	Tcl_SetResult(interp, buff, TCL_VOLATILE);
+	return TCL_OK;
+    } else {
+	Tcl_AppendResult(interp, "bad option \"", argv[1],
+	    "\": must be cget, configure or refcount", (char *) NULL);
+	return TCL_ERROR;
     }
-
-  error:
-
-    Tcl_AppendResult(interp, "bad option \"", argv[1],
-	"\": must be cget or configure", (char *) NULL);
-    return TCL_ERROR;
 }
 
 /*
@@ -1216,11 +1030,11 @@ ImgXpmGet(tkwin, masterData)
     instancePtr->masterPtr = masterPtr;
     instancePtr->tkwin = tkwin;
     instancePtr->pixmap = None;
-    instancePtr->mask = None;
-    instancePtr->gc = None;
     instancePtr->nextPtr = masterPtr->instancePtr;
     instancePtr->colors = NULL;
     masterPtr->instancePtr = instancePtr;
+
+    TixpInitPixmapInstance(masterPtr, instancePtr);
     ImgXpmConfigureInstance(instancePtr);
 
     /*
@@ -1268,30 +1082,8 @@ ImgXpmDisplay(clientData, display, drawable, imageX, imageY, width,
     int drawableX, drawableY;	/* Coordinates within drawable that
 				 * correspond to imageX and imageY. */
 {
-    PixmapInstance *instancePtr = (PixmapInstance *) clientData;
-
-    /*
-     * If there's no graphics context, it means that an error occurred
-     * while creating the image instance so it can't be displayed.
-     */
-
-    if (instancePtr->gc == None) {
-	return;
-    }
-
-    /*
-     * We always use masking: modify the mask origin within
-     * the graphics context to line up with the image's origin.
-     * Then draw the image and reset the clip origin, if there's
-     * a mask.
-     */
-
-    XSetClipOrigin(display, instancePtr->gc, drawableX - imageX,
-	drawableY - imageY);
-    XCopyArea(display, instancePtr->pixmap, drawable, instancePtr->gc,
-	imageX, imageY, (unsigned) width, (unsigned) height,
-	drawableX, drawableY);
-    XSetClipOrigin(display, instancePtr->gc, 0, 0);
+    TixpXpmDisplay(clientData, display, drawable, imageX, imageY, width,
+	height, drawableX, drawableY);
 }
 
 /*
@@ -1330,14 +1122,10 @@ ImgXpmFree(clientData, display)
      * the instance structure.
      */
     if (instancePtr->pixmap != None) {
-	XFreePixmap(display, instancePtr->pixmap);
-    }
-    if (instancePtr->mask != None) {
-	XFreePixmap(display, instancePtr->mask);
-    }
-    if (instancePtr->gc != None) {
-	Tk_FreeGC(display, instancePtr->gc);
+	Tk_FreePixmap(display, instancePtr->pixmap);
     }
+    TixpXpmFreeInstanceData(instancePtr, 1);
+
     if (instancePtr->colors != NULL) {
 	int i;
 	for (i=0; i<instancePtr->masterPtr->ncolors; i++) {
@@ -1397,7 +1185,9 @@ ImgXpmDelete(masterData)
     }
     if (masterPtr->isDataAlloced && masterPtr->data != NULL) {
 	ckfree((char*)masterPtr->data);
+	masterPtr->data = NULL;
     }
+
     Tk_FreeOptions(configSpecs, (char *) masterPtr, (Display *) NULL, 0);
     ckfree((char *) masterPtr);
 }
