@@ -10,51 +10,17 @@
  *
  */
 
-#include "tkPort.h"
-#include "tkInt.h"
+#include <tixPort.h>
 #include <tixInt.h>
-#include "tixPatchLevel.h"
 
-#ifdef  _Windows
-#ifndef _WINDOWS
-#define _WINDOWS
-#endif
+#ifdef ITCL_2
+#include <itcl.h>
 #endif
 
 #ifdef _WINDOWS
 #include <tkWinInt.h>
 #endif
 
-extern TIX_DECLARE_CMD(Tix_CallMethodCmd);
-extern TIX_DECLARE_CMD(Tix_ChainMethodCmd);
-extern TIX_DECLARE_CMD(Tix_ClassCmd);
-extern TIX_DECLARE_CMD(Tix_DoWhenIdleCmd);
-extern TIX_DECLARE_CMD(Tix_DoWhenMappedCmd);
-extern TIX_DECLARE_CMD(Tix_FileCmd);
-extern TIX_DECLARE_CMD(Tix_FlushXCmd);
-extern TIX_DECLARE_CMD(Tix_FormCmd);
-extern TIX_DECLARE_CMD(Tix_GridCmd);
-extern TIX_DECLARE_CMD(Tix_GeometryRequestCmd);
-extern TIX_DECLARE_CMD(Tix_Get3DBorderCmd);
-extern TIX_DECLARE_CMD(Tix_GetBooleanCmd);
-extern TIX_DECLARE_CMD(Tix_GetIntCmd);
-extern TIX_DECLARE_CMD(Tix_GetMethodCmd);
-extern TIX_DECLARE_CMD(Tix_HListCmd);
-extern TIX_DECLARE_CMD(Tix_HandleOptionsCmd);
-extern TIX_DECLARE_CMD(Tix_InputOnlyCmd);
-extern TIX_DECLARE_CMD(Tix_ItemStyleCmd);
-extern TIX_DECLARE_CMD(Tix_ManageGeometryCmd);
-extern TIX_DECLARE_CMD(Tix_MapWindowCmd);
-extern TIX_DECLARE_CMD(Tix_MoveResizeWindowCmd);
-extern TIX_DECLARE_CMD(Tix_NoteBookFrameCmd);
-extern TIX_DECLARE_CMD(Tix_RaiseWindowCmd);
-extern TIX_DECLARE_CMD(Tix_ShellInputCmd);
-extern TIX_DECLARE_CMD(Tix_StringSubCmd);
-extern TIX_DECLARE_CMD(Tix_TListCmd);
-extern TIX_DECLARE_CMD(Tix_TmpLineCmd);
-extern TIX_DECLARE_CMD(Tix_UnmapWindowCmd);
-extern TIX_DECLARE_CMD(Tix_MwmCmd);
-
 static Tix_TclCmd commands[] = {
     /*
      * Commands that are part of the intrinsics:
@@ -65,6 +31,7 @@ static Tix_TclCmd commands[] = {
     {"tixDisplayStyle",         Tix_ItemStyleCmd},
     {"tixDoWhenIdle",           Tix_DoWhenIdleCmd},
     {"tixDoWhenMapped",         Tix_DoWhenMappedCmd},
+    {"tixFalse",                Tix_FalseCmd},
     {"tixFile",                 Tix_FileCmd},
     {"tixFlushX",           	Tix_FlushXCmd},
     {"tixForm",                 Tix_FormCmd},
@@ -89,17 +56,23 @@ static Tix_TclCmd commands[] = {
     {"tixRaiseWindow",          Tix_RaiseWindowCmd},
     {"tixShellInput",           Tix_ShellInputCmd},
     {"tixStringSub",		Tix_StringSubCmd},
+    {"tixStrEq",		Tix_StrEqCmd},
     {"tixTmpLine",              Tix_TmpLineCmd},
+    {"tixTrue",                 Tix_TrueCmd},
     {"tixUnmapWindow",          Tix_UnmapWindowCmd},
     {"tixWidgetClass",          Tix_ClassCmd},
     {"tixWidgetDoWhenIdle",     Tix_DoWhenIdleCmd},
 
+#if ((TCL_MAJOR_VERSION == 7) && (TCL_MINOR_VERSION == 5))
+    {"tixConsoleInit",		Tix_ConsoleInitCmd},
+#endif
+
 #ifndef TIX_VERSION_4_0_x
     {"tixTList",     		Tix_TListCmd},
     {"tixGrid",     		Tix_GridCmd},
 #endif
 
-    {(char *) NULL,		(int (*)()) NULL}
+    {(char *) NULL,		(Tix_CmdProc)NULL}
 };
 
 typedef struct {
@@ -114,22 +87,39 @@ typedef struct {
 
 static OptionStruct tixOption;
 
+/*
+ * TIX_DEF_FONTSET and TIX_DEF_SCHEME should have been defined in the
+ * Makefile by the configure script. We define them here just in case
+ * the configure script failed to determine the proper values.
+ */
+
+#ifndef TIX_DEF_FONTSET
+#define TIX_DEF_FONTSET "14Point"
+#endif
+
+#ifndef TIX_DEF_SCHEME
+#define TIX_DEF_SCHEME "TixGray"
+#endif
+
+
 #define DEF_TIX_TOOLKIT_OPTION_BETA		"1"
 #define DEF_TIX_TOOLKIT_OPTION_BINDING		"Motif"
 #define DEF_TIX_TOOLKIT_OPTION_DEBUG		"1"
-#define DEF_TIX_TOOLKIT_OPTION_FONTSET		"14Point"
+#define DEF_TIX_TOOLKIT_OPTION_FONTSET		TIX_DEF_FONTSET
 #define DEF_TIX_TOOLKIT_OPTION_LIBRARY		""
-#define DEF_TIX_TOOLKIT_OPTION_SCHEME		"TixGray"
+#define DEF_TIX_TOOLKIT_OPTION_SCHEME		TIX_DEF_SCHEME
 #define DEF_TIX_TOOLKIT_OPTION_SCHEME_PRIORITY	"79"
 
 static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BOOLEAN, "-beta", "tixBeta", "TixBeta",
-       DEF_TIX_TOOLKIT_OPTION_BETA, Tk_Offset(OptionStruct, isBeta), 0},
+       DEF_TIX_TOOLKIT_OPTION_BETA, Tk_Offset(OptionStruct, isBeta),
+       0},
     {TK_CONFIG_STRING, "-binding", "binding", "TixBinding",
        DEF_TIX_TOOLKIT_OPTION_BINDING, Tk_Offset(OptionStruct, binding),
        0},
     {TK_CONFIG_BOOLEAN, "-debug", "tixDebug", "TixDebug",
-       DEF_TIX_TOOLKIT_OPTION_DEBUG, Tk_Offset(OptionStruct, isDebug), 0},
+       DEF_TIX_TOOLKIT_OPTION_DEBUG, Tk_Offset(OptionStruct, isDebug),
+       0},
     {TK_CONFIG_STRING, "-fontset", "tixFontSet", "TixFontSet",
        DEF_TIX_TOOLKIT_OPTION_FONTSET, Tk_Offset(OptionStruct, fontSet),
        0},
@@ -142,7 +132,7 @@ static Tk_ConfigSpec configSpecs[] = {
        0},
     {TK_CONFIG_STRING, "-tixlibrary", "tixLibrary", "TixLibrary",
        DEF_TIX_TOOLKIT_OPTION_LIBRARY, Tk_Offset(OptionStruct, tixlibrary),
-       0},
+       TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
        (char *) NULL, 0, 0}
 };
@@ -151,7 +141,7 @@ static Tk_ConfigSpec configSpecs[] = {
 #ifndef _WINDOWS
 #define TIX_LIBRARY "/usr/local/lib/tix"
 #else
-#define TIX_LIBRARY "../library"
+#define TIX_LIBRARY "../../library"
 #endif
 #endif
 
@@ -162,6 +152,9 @@ static Tk_ConfigSpec configSpecs[] = {
  *----------------------------------------------------------------------
  */
 Tk_Uid tixNormalUid   = (Tk_Uid)NULL;
+Tk_Uid tixCellUid     = (Tk_Uid)NULL;
+Tk_Uid tixRowUid      = (Tk_Uid)NULL;
+Tk_Uid tixColumnUid   = (Tk_Uid)NULL;
 Tk_Uid tixDisabledUid = (Tk_Uid)NULL;
 
 /*----------------------------------------------------------------------
@@ -177,8 +170,8 @@ extern Tix_DItemInfo tix_WindowItemType;
 extern Tix_DItemInfo tix_ImageItemType;
 
 static int		ParseToolkitOptions _ANSI_ARGS_((Tcl_Interp * interp));
-static int 		mwmProtocolHandler _ANSI_ARGS_((ClientData clientData,
-			    XEvent *eventPtr));
+extern int 		TixMwmProtocolHandler _ANSI_ARGS_((
+			    ClientData clientData, XEvent *eventPtr));
 static int		Tix_Init_Internal _ANSI_ARGS_((Tcl_Interp * interp,
 			     int doSource));
 int 			Tix_EtInit _ANSI_ARGS_((Tcl_Interp * interp));
@@ -206,7 +199,7 @@ ParseToolkitOptions(interp)
     tixOption.schemePriority = NULL;
 
     /*
-	  * The toolkit options may be set in the resources of the main window
+     * The toolkit options may be set in the resources of the main window
      */
     if (Tk_ConfigureWidget(interp, Tk_MainWindow(interp), configSpecs,
 	    0, 0, (char *) &tixOption, 0) != TCL_OK) {
@@ -223,15 +216,33 @@ ParseToolkitOptions(interp)
     sprintf(buff, "%d", tixOption.isDebug);
     Tcl_SetVar2(interp, "tix_priv", "-debug", buff, flag);
 
-    if (strlen(tixOption.tixlibrary) == 0) {
-	/* Set up the TCL variable "tix_library" according to the environment
+    if (tixOption.tixlibrary == 0 || strlen(tixOption.tixlibrary) == 0) {
+	/*
+	 * Set up the TCL variable "tix_library" according to the environment
 	 * variable.
 	 */
-	 tixOption.tixlibrary= getenv("TIX_LIBRARY");
-	 if (tixOption.tixlibrary == NULL) {
-	     tixOption.tixlibrary = TIX_LIBRARY;
-	 }
-     }
+	if (tixOption.tixlibrary != NULL) {
+	    ckfree((char*)tixOption.tixlibrary);
+	}
+
+	tixOption.tixlibrary = (char*)getenv("TIX_LIBRARY");
+	if (tixOption.tixlibrary == NULL) {
+	    tixOption.tixlibrary = TIX_LIBRARY;
+	}
+	Tcl_SetVar2(interp, "tix_priv", "-libdir",  
+	 	tixOption.tixlibrary, flag);
+    } else {
+	Tcl_SetVar2(interp, "tix_priv", "-libdir",  
+	 	tixOption.tixlibrary, flag);
+	ckfree((char*)tixOption.tixlibrary);
+    }
+
+    /*
+     * tixOption.tixlibrary may not be allocated by Tk_ConfigureWidget().
+     * We have already freed it (if necessary). We set it to NULL so
+     * that Tk_FreeOptions() won't try to free it.
+     */
+    tixOption.tixlibrary = NULL;
 
     Tcl_SetVar2(interp, "tix_priv", "-binding",
 	tixOption.binding,    		flag);
@@ -241,35 +252,13 @@ ParseToolkitOptions(interp)
 	tixOption.scheme,     		flag);
     Tcl_SetVar2(interp, "tix_priv", "-schemepriority",
 	tixOption.schemePriority,     flag);
-    Tcl_SetVar2(interp, "tix_priv", "-libdir",  
-	tixOption.tixlibrary, 		flag);
+
+    Tk_FreeOptions(configSpecs, (char *)&tixOption,
+	Tk_Display(Tk_MainWindow(interp)), 0);
 
     return TCL_OK;
 }
-
-static int mwmProtocolHandler(clientData, eventPtr)
-    ClientData clientData;
-    XEvent *eventPtr;
-{
-    TkWindow *winPtr;
-    Window handlerWindow;
-
-    if (eventPtr->type != ClientMessage) {
-	return 0;
-    }
-
-	 handlerWindow = eventPtr->xany.window;
-    winPtr = (TkWindow *) Tk_IdToWindow(eventPtr->xany.display, handlerWindow);
-    if (winPtr != NULL) {
-	if (eventPtr->xclient.message_type ==
-	    Tk_InternAtom((Tk_Window) winPtr,"_MOTIF_WM_MESSAGES")) {
-	    TkWmProtocolEventProc(winPtr, eventPtr);
-	    return 1;
-	}
-    }
-    return 0;
-}
-
+
 /*----------------------------------------------------------------------
  * Tix_Init_Internal() --
  *
@@ -279,6 +268,7 @@ static int mwmProtocolHandler(clientData, eventPtr)
  *	applicattion.
  *----------------------------------------------------------------------
  */
+
 static int
 Tix_Init_Internal(interp, doSource)
 	 Tcl_Interp * interp;
@@ -286,43 +276,105 @@ Tix_Init_Internal(interp, doSource)
 {
     Tk_Window topLevel;
     char * appName;
+    static int globalInitialized = 0;
+
+    /*
+     * This procedure may be called  several times for several
+     * interpreters. Since some global variables are shared by
+     * all of the interpreters, we initialize these variables only
+     * once. The variable "globalInitialized" keeps track of this
+     */
 
     extern Tk_ImageType tixPixmapImageType;
     extern Tk_ImageType tixCompoundImageType;
 
+
+#ifdef TCL_7_5_OR_LATER
+    /*
+     * The new package mechanism, available in Tcl7.5 or later
+     */
+    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
+	return TCL_ERROR;
+    }
+    if (Tcl_PkgRequire(interp, "Tk", TK_VERSION, 1) == NULL) {
+	return TCL_ERROR;
+    }
+#ifdef ITCL_2
+    if (Tcl_PkgRequire(interp, "Itcl", ITCL_VERSION, 0) == NULL) {
+	return TCL_ERROR;
+    }
+    if (Tcl_PkgRequire(interp, "Itk", ITCL_VERSION, 0) == NULL) {
+	return TCL_ERROR;
+    }
+#endif
+/*
+ * // This is now done in Init.tcl
+ *   if (Tcl_PkgProvide(interp, "Tix", TIX_VERSION) != TCL_OK) {
+ *	return TCL_ERROR;
+ *   }
+ */
+#endif
+
     topLevel = Tk_MainWindow(interp);
 
-    /* Initialize some global variables */
-    tixNormalUid   = Tk_GetUid("normal");
-    tixDisabledUid = Tk_GetUid("disabled");
-    
-    /* This is for tixMwm command */
-    Tk_CreateGenericHandler(mwmProtocolHandler, NULL);
+    if (!globalInitialized) {
+	globalInitialized = 1;
 
-    /* Set the "tix_version" variable */
+	/*
+	 * Initialize the global variables shared by all interpreters
+	 */
+	tixNormalUid   = Tk_GetUid("normal");
+	tixCellUid     = Tk_GetUid("cell");
+	tixRowUid      = Tk_GetUid("row");
+	tixColumnUid   = Tk_GetUid("column");
+	tixDisabledUid = Tk_GetUid("disabled");
+
+#ifndef _WINDOWS
+	/* This is for tixMwm command */
+	Tk_CreateGenericHandler(TixMwmProtocolHandler, NULL);
+#endif
+
+	/* Initialize the image readers */
+	Tk_CreateImageType(&tixPixmapImageType);
+	Tk_CreateImageType(&tixCompoundImageType);
+
+	/* Initialize the display item types */
+	Tix_AddDItemType(&tix_ImageTextItemType);
+	Tix_AddDItemType(&tix_TextItemType);
+	Tix_AddDItemType(&tix_WindowItemType);
+	Tix_AddDItemType(&tix_ImageItemType);
+
+	/*
+	 * Initializes all the Tix built-in bitmaps.
+	 */
+#define Et_Interp interp
+#include "tixBitmaps.h"
+
+    }
+    else {
+	/*
+	 * This variable is used in the __tixInit procedure.
+	 */
+	Tcl_SetVar2(interp, "tix_priv", "slaveInterp", "", TCL_GLOBAL_ONLY);
+    }
+
+    /*
+     * Initialize the per-interpreter variables
+     */
+
+    /*  Set the "tix_version" variable */
     Tcl_SetVar(interp, "tix_version",    TIX_VERSION,    TCL_GLOBAL_ONLY);
     Tcl_SetVar(interp, "tix_patchLevel", TIX_PATCHLEVEL, TCL_GLOBAL_ONLY);
+    Tcl_SetVar(interp, "tix_release",    TIX_RELEASE,    TCL_GLOBAL_ONLY);
 
     /* Initialize the Tix commands */
     Tix_CreateCommands(interp, commands, (ClientData) topLevel,
-	(void (*)()) NULL);
-
-    /* Initialize the image readers */
-#ifndef _WINDOWS
-    Tk_CreateImageType(&tixPixmapImageType);
-#endif
-    Tk_CreateImageType(&tixCompoundImageType);
+	(void (*)_ANSI_ARGS_((ClientData))) NULL);
 
 #ifdef _WINDOWS
     Tcl_GlobalEval(interp, "set tixPriv(isWindows) 1");
 #endif
 
-    /* Initialize the display item types */
-    Tix_AddDItemType(&tix_ImageTextItemType);
-    Tix_AddDItemType(&tix_TextItemType);
-    Tix_AddDItemType(&tix_WindowItemType);
-    Tix_AddDItemType(&tix_ImageItemType);
-
     /* Parse options database for fontSets, schemes, etc */
     if (ParseToolkitOptions(interp) == TCL_ERROR) {
 	return TCL_ERROR;
@@ -333,56 +385,190 @@ Tix_Init_Internal(interp, doSource)
     }
 
     if (doSource) {
-	char stuff[100];
 
-	/* Load the Tix library */
-	if (Tix_LoadTclLibrary(interp, "TIX_LIBRARY", "tix_library", 
-	    "Init.tcl",	TIX_LIBRARY, appName) != TCL_OK) {
+	if (TixLoadLibrary(interp) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 
-	/* Let's check whether the TIX_LIBRARY variable is set to 
-	 * a pre-4.0.2 version of Tix. (All 4.0.2+ version will correctly
-	 * identify their own versions and will print out warning messages
-	 * if the version of the binary does not match with the script library
+	/*
+	 * Check whether the TIX_LIBRARY variable is set to a
+	 * pre-4.0.2 version of Tix. (All 4.0.2+ version will
+	 * correctly identify their own versions and will print out
+	 * warning messages if the version of the binary does not
+	 * match with the script library
 	 */
-	strcpy(stuff, "tixScriptVersion");
-	if (Tcl_GlobalEval(interp, stuff) == TCL_ERROR) {
+	if (Tcl_GlobalEval(interp, "tixScriptVersion") != TCL_OK) {
 	    fprintf(stderr, 
 		"Warning: Tix script library version (pre 4.0.2)\n");
+	    fprintf(stderr, "  in \"%s\"\n", Tcl_GetVar(interp, "tix_library",
+		    TCL_GLOBAL_ONLY));
 	    fprintf(stderr, "  does not match binary version (%s).\n",
 		TIX_PATCHLEVEL);
 	    fprintf(stderr, "  Please check your TIX_LIBRARY environment ");
 	    fprintf(stderr, "variable and your Tix installation.\n");
 	    Tcl_ResetResult(interp);
 	}
-	return TCL_OK;
-    } else {
-	Tcl_SetVar(interp, "tix_library", "nowhere", TCL_GLOBAL_ONLY);
-	return TCL_OK;
-    }
-}
 
-/* Tix_Init --
+	if (Tcl_GlobalEval(interp, "__tixInit") != TCL_OK) {
+	    return TCL_ERROR;
+	}
+    } else {
+	Tcl_SetVar(interp, "tix_library", "", TCL_GLOBAL_ONLY);
+    }
+
+
+    return TCL_OK;
+}
+
+/*----------------------------------------------------------------------
+ * Tix_Init --
  *
  * 	This is the function to call in your Tcl_AppInit() function
+ *
+ *----------------------------------------------------------------------
  */
+
 int
 Tix_Init(interp)
     Tcl_Interp * interp;
 {
-    return Tix_Init_Internal(interp, 1);
-}
+    int code = Tix_Init_Internal(interp, 1);
 
-/* Tix_EtInit --
+#ifdef _WINDOWS
+    if (code != TCL_OK) {
+	char * errorInfo;
+
+	errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
+	if (errorInfo == NULL) {
+	    Tcl_SetVar(interp, "errorInfo", "unknown error", TCL_GLOBAL_ONLY);
+	}
+	Tix_GlobalVarEval(interp,
+	    "toplevel .err; ",
+		"bind .err <Destroy> {set err_ok 1}; ",
+		"frame .err.f; pack .err.f -side bottom -fill both; "
+	    "button .err.f.i -text Ignore -width 6 -command {set err_ok 1}; ",
+	    "button .err.f.e -text Exit   -width 6 -command {exit}; ",
+	    "pack .err.f.i -side left -padx 4 -pady 4;"
+	    "pack .err.f.e -side left -padx 4 -pady 4; "
+	    "text .err.text -width 70 -wrap none -height 5; "
+	    "pack .err.text -side top -expand yes -fill both; "
+	    ".err.text insert end $errorInfo; ",
+	    "tkwait variable err_ok; ",
+	    "catch {destroy .err}; ",
+	    NULL);
+    }
+#endif
+    return code;
+}
+
+/*----------------------------------------------------------------------
+ * TixInitSam --
  *
- * 	This takes special care when you initialize the Tix
- *	library from an ET application.
+ * 	This takes special care when you initialize the Tix library
+ * 	to run in stand-alone mode.
+ *----------------------------------------------------------------------
  */
-int Tix_EtInit(interp)
-	 Tcl_Interp * interp;
+int TixInitSam(interp)
+    Tcl_Interp * interp;
 {
     return Tix_Init_Internal(interp, 0);
 }
+
+/*----------------------------------------------------------------------
+ * Tix_SafeInit --
+ *
+ * 	This is the function to call in your Tcl_AppInit() function
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tix_SafeInit(interp)
+    Tcl_Interp * interp;
+{
+    Tcl_SetVar2(interp, "tix_priv", "isSafe", "1", TCL_GLOBAL_ONLY);
+    return Tix_Init(interp);
+}
+
+/*
+ *----------------------------------------------------------------------
+ * TixLoadLibrary --
+ *
+ *	Loads the Tix library.
+ *
+ * Results:
+ *	Standard Tcl result.
+ *
+ * Side effects:
+ *	Tix gets initialized.
+ *----------------------------------------------------------------------
+ */
+
+static char *initScript =
+"if [catch {file join a a}] {\n\
+    proc tixFileJoin {args} {\n\
+	set p [join $args /]\n\
+	regsub -all {/+} $p / p\n\
+	return $p\n\
+    }\n\
+} else {\n\
+    proc tixFileJoin {args} {\n\
+	return [eval file join $args]\n\
+   }\n\
+}\n\
+\n\
+proc init {} {\n\
+    global tix_library tix_version tix_patchLevel env\n\
+    rename init {}\n\
+    set dirs {}\n\
+    if [info exists env(TIX_LIBRARY)] {\n\
+	lappend dirs $env(TIX_LIBRARY)\n\
+    }\n\
+    if [string match {*[ab]*} $tix_patchLevel] {\n\
+	set lib tix$tix_patchLevel\n\
+	set Lib Tix$tix_patchLevel\n\
+    } else {\n\
+	set lib tix$tix_version\n\
+	set Lib Tix$tix_version\n\
+    }\n\
+    catch {\n\
+        # [pwd] may not work inside safe Tcl\n\
+        set p [pwd]\n\
+	lappend dirs [tixFileJoin $p library]\n\
+        set p [file dirname $p]\n\
+	lappend dirs [tixFileJoin $p library]\n\
+        set p [file dirname $p]\n\
+	lappend dirs [tixFileJoin $p library]\n\
+    }\n\
+    set instDir [file dirname [info library]]\n\
+    lappend dirs [tixFileJoin $instDir $lib]\n\
+    lappend dirs [tixFileJoin [tixFileJoin $instDir lib] $lib]\n\
+    catch {\n\
+    lappend dirs [tixFileJoin [tixFileJoin [file dirname [file dirname [info nameofexecutable]]] lib] $lib]\n\
+    }\n\
+    lappend dirs [tixFileJoin [tixFileJoin [file dirname [file dirname [info library]]] $lib] library]\n\
+    lappend dirs [tixFileJoin [tixFileJoin [file dirname [file dirname [info library]]] $Lib] library]\n\
+    foreach i $dirs {\n\
+	set tix_library $i\n\
+	if ![catch {uplevel #0 source [tixFileJoin $i Init.tcl]} err] {\n\
+	    return\n\
+	} else {\n\
+	    if [info exists env(TIX_DEBUG)] {\n\
+		puts $err\n\
+	    }\n\
+        }\n\
+    }\n\
+    set msg \"Can't find a usable Init.tcl in the following directories: \n\"\n\
+    append msg \"    $dirs\n\"\n\
+    append msg \"This probably means that Tix wasn't installed properly.\n\"\n\
+    error $msg\n\
+}\n\
+init";
 
 
+int
+TixLoadLibrary(interp)
+    Tcl_Interp * interp;
+{
+    return Tcl_Eval(interp, initScript);
+}
