@@ -19,11 +19,26 @@
  *
  */
 #include <tclInt.h>
-#include <tkPort.h>
 #include <tk.h>
-#include <tix.h>
+#include <tixPort.h>
+#include <tixInt.h>
 
-Tcl_HashTable methodTable;
+#define GetMethodTable(interp) (_TixGetHashTable(interp, "tixMethodTab", MethodTableDeleteProc))
+
+static int		Tix_CallMethodByContext _ANSI_ARGS_((
+			    Tcl_Interp * interp, char * context,
+			    char * widRec, char * method, int argc,
+			    char ** argv));
+static void		Tix_RestoreContext _ANSI_ARGS_((
+			    Tcl_Interp * interp, char * widRec,
+			    char * oldContext));
+static void		Tix_SetContext _ANSI_ARGS_((
+			    Tcl_Interp * interp, char * widRec,
+			    char * newContext));
+static char *		Tix_SaveContext _ANSI_ARGS_((Tcl_Interp * interp,
+			    char * widRec));
+static void		MethodTableDeleteProc _ANSI_ARGS_((
+			    ClientData clientData, Tcl_Interp *interp));
 
 /*
  *
@@ -45,6 +60,7 @@ TIX_DEFINE_CMD(Tix_CallMethodCmd)
     }
  
     if ((context = GET_RECORD(interp, widRec, "className")) == NULL) {
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "invalid object reference \"", widRec,
 	    "\"", (char*)NULL);
 	return TCL_ERROR;
@@ -142,6 +158,7 @@ TIX_DEFINE_CMD(Tix_GetMethodCmd)
 
     if (newContext) {
 	cmdName = Tix_GetMethodFullName(newContext, method);
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, cmdName, NULL);
 	ckfree(cmdName);
     } else {
@@ -158,7 +175,8 @@ TIX_DEFINE_CMD(Tix_GetMethodCmd)
  * the method. This class must be the same as the class "context" or
  * a superclass of the class "context".
  */
-char * Tix_FindMethod(interp, context, method)
+char *
+Tix_FindMethod(interp, context, method)
     Tcl_Interp * interp;
     char * context;
     char * method;
@@ -167,15 +185,9 @@ char * Tix_FindMethod(interp, context, method)
     int    	isNew;
     char      * key;
     Tcl_HashEntry *hashPtr;
-    static inited = 0;
-
-    if (!inited) {
-	Tcl_InitHashTable(&methodTable, TCL_STRING_KEYS);
-	inited = 1;
-    }
 
     key = Tix_GetMethodFullName(context, method);
-    hashPtr = Tcl_CreateHashEntry(&methodTable, key, &isNew);
+    hashPtr = Tcl_CreateHashEntry(GetMethodTable(interp), key, &isNew);
     ckfree(key);
 
     if (!isNew) {
@@ -195,8 +207,9 @@ char * Tix_FindMethod(interp, context, method)
 	}
 
 	if (theContext != NULL) {
-	    /* theContext may point to the stack. We have to put it
-	     * in some more permanent place 
+	    /*
+	     * theContext may point to the stack. We have to put it
+	     * in some more permanent place.
 	     */
 	    theContext = (char*)strdup(theContext);
 	}
@@ -222,11 +235,13 @@ int Tix_CallMethod(interp, context, widRec, method, argc, argv)
 {
     char * targetContext;
 
-    if (targetContext =  Tix_FindMethod(interp, context, method)) {
+    targetContext = Tix_FindMethod(interp, context, method);
+    if (targetContext != NULL) {
 	return Tix_CallMethodByContext(interp, targetContext, widRec, method,
 	    argc, argv);
     }
     else {
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "cannot call method \"", method,
 	    "\" for context \"", context, "\".", (char*)NULL);
 	Tcl_SetVar(interp, "errorInfo", interp->result, TCL_GLOBAL_ONLY);
@@ -245,13 +260,14 @@ int Tix_CallMethod(interp, context, widRec, method, argc, argv)
 /* save the old context: calling a method of a superclass will
  * change the context of a widget.
  */
-char * Tix_SaveContext(interp, widRec)
+static char * Tix_SaveContext(interp, widRec)
     Tcl_Interp * interp;
     char * widRec;
 {
     char * context;
 
     if ((context = GET_RECORD(interp, widRec, "context")) == NULL) {
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "invalid object reference \"", widRec,
 	    "\"", (char*)NULL);
 	return NULL;
@@ -261,7 +277,7 @@ char * Tix_SaveContext(interp, widRec)
     }
 }
 
-void Tix_RestoreContext(interp, widRec, oldContext)
+static void Tix_RestoreContext(interp, widRec, oldContext)
     Tcl_Interp * interp;
     char * widRec;
     char * oldContext;
@@ -270,7 +286,7 @@ void Tix_RestoreContext(interp, widRec, oldContext)
     ckfree(oldContext);
 }
 
-void Tix_SetContext(interp, widRec, newContext)
+static void Tix_SetContext(interp, widRec, newContext)
     Tcl_Interp * interp;
     char * widRec;
     char * newContext;
@@ -286,6 +302,7 @@ char * Tix_GetContext(interp, widRec)
     char * context;
 
     if ((context = GET_RECORD(interp, widRec, "context")) == NULL) {
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "invalid object reference \"", widRec,
 	    "\"", (char*)NULL);
 	return NULL;
@@ -302,6 +319,7 @@ int Tix_SuperClass(interp, class, superClass_ret)
     char * superclass;
 
     if ((superclass = GET_RECORD(interp, class, "superClass")) == NULL) {
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "invalid class \"", class,
 	    "\"; ", (char*)NULL);
 	return TCL_ERROR;
@@ -329,15 +347,15 @@ char * Tix_GetMethodFullName(context, method)
     buff = (char*)ckalloc(max * sizeof(char));
 
     strcpy(buff, context);
-    strcpy(buff+conLen, "::");
-    strcpy(buff+conLen+2, method);
+    strcpy(buff+conLen, ":");
+    strcpy(buff+conLen+1, method);
 
     return buff;
 }
 
-#undef ITCL_20
+#undef ITCL_2
 
-#ifndef ITCL_20
+#ifndef ITCL_2
 
 #define Tix_GetCommandInfo Tcl_GetCommandInfo
 
@@ -420,7 +438,7 @@ int Tix_ExistMethod(interp, context, method)
 /* %% There is a dirty version that uses the old argv, without having to
  * malloc a new argv.
  */
-int Tix_CallMethodByContext(interp, context, widRec, method, argc, argv)
+static int Tix_CallMethodByContext(interp, context, widRec, method, argc, argv)
     Tcl_Interp * interp;
     char * context;
     char * widRec;
@@ -456,14 +474,18 @@ int Tix_CallMethodByContext(interp, context, widRec, method, argc, argv)
     return result;
 }
 
-#ifndef ITCL_20
+#ifndef ITCL_2
 
 #define Tix_GlobalEvalArgv(interp, cmdInfoPtr, argc, argv) \
     (*(cmdInfoPtr)->proc)((cmdInfoPtr)->clientData, interp, argc, argv)
 
 #else
 
-int Tix_GlobalEvalArgv(interp, cmdInfoPtr, argc, argv)
+EXTERN int		Tix_GlobalEvalArgv _ANSI_ARGS_((Tcl_Interp * interp,
+			    Tcl_CmdInfo * cmdInfoPtr, int argc));
+
+int
+Tix_GlobalEvalArgv(interp, cmdInfoPtr, argc, argv)
     Tcl_Interp * interp;
     Tcl_CmdInfo * cmdInfoPtr;
     int argc;
@@ -489,7 +511,7 @@ int Tix_GlobalEvalArgv(interp, cmdInfoPtr, argc, argv)
     iPtr->varFramePtr = savedVarFramePtr;
     return result;
 }
-#endif /* ITCL_20 */
+#endif /* ITCL_2 */
 
 int Tix_EvalArgv(interp, argc, argv)
     Tcl_Interp * interp;
@@ -505,6 +527,7 @@ int Tix_EvalArgv(interp, argc, argv)
 	 * This comand is not defined yet -- looks like we have to auto-load it
 	 */
 	if (!Tix_GetCommandInfo(interp, "auto_load", &cmdInfo)) {
+	    Tcl_ResetResult(interp);
 	    Tcl_AppendResult(interp, "cannot execute command \"auto_load\"",
 		NULL);
 	    return TCL_ERROR;
@@ -518,7 +541,8 @@ int Tix_EvalArgv(interp, argc, argv)
 	}
 
 	if (!Tix_GetCommandInfo(interp, argv[0], &cmdInfo)) {
-	    Tcl_AppendResult(interp, "cannot auto-load command \"",
+	    Tcl_ResetResult(interp);
+	    Tcl_AppendResult(interp, "cannot autoload command \"",
 		argv[0], "\"",NULL);
 	    return TCL_ERROR;
 	}
@@ -527,7 +551,8 @@ int Tix_EvalArgv(interp, argc, argv)
     return Tix_GlobalEvalArgv(interp, &cmdInfo, argc, argv);
 }
 
-char * Tix_FindPublicMethod(interp, cPtr, method)
+char *
+Tix_FindPublicMethod(interp, cPtr, method)
     Tcl_Interp * interp;
     TixClassRecord * cPtr;
     char * method;
@@ -543,5 +568,43 @@ char * Tix_FindPublicMethod(interp, cPtr, method)
     }
     return 0;
 }
+
+/*
+ *----------------------------------------------------------------------
+ * MethodTableDeleteProc --
+ *
+ *	This procedure is called when the interp is about to
+ *	be deleted. It cleans up the hash entries and destroys the hash
+ *	table.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	All class method contexts are deleted for this interpreter.
+ *----------------------------------------------------------------------
+ */
 
+static void
+MethodTableDeleteProc(clientData, interp)
+    ClientData clientData;
+    Tcl_Interp *interp;
+{
+    Tcl_HashTable * methodTablePtr = (Tcl_HashTable*)clientData;
+    Tcl_HashSearch hashSearch;
+    Tcl_HashEntry *hashPtr;
+    char * context;
 
+    for (hashPtr = Tcl_FirstHashEntry(methodTablePtr, &hashSearch);
+	 hashPtr;
+	 hashPtr = Tcl_NextHashEntry(&hashSearch)) {
+
+	context = (char*)Tcl_GetHashValue(hashPtr);
+	if (context) {
+	    ckfree(context);
+	}
+	Tcl_DeleteHashEntry(hashPtr);
+    }
+    Tcl_DeleteHashTable(methodTablePtr);
+    ckfree((char*)methodTablePtr);
+}
