@@ -3,13 +3,11 @@
  *
  *	This module implements "TList" widgets.
  *
- * Copyright (c) 1993-1999 Ioi Kim Lam.
- * Copyright (c) 2000      Tix Project Group.
+ *
+ * Copyright (c) 1996, Expert Interface Technologies
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * $Id: tixTList.c,v 1.5 2004/03/28 02:44:57 hobbs Exp $
  */
 
 #include <tixPort.h>
@@ -188,13 +186,13 @@ static Tix_ListInfo entListInfo = {
 static void		WidgetCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
 static int		WidgetConfigure _ANSI_ARGS_((Tcl_Interp *interp,
-			    WidgetPtr wPtr, int argc, CONST84 char **argv,
+			    WidgetPtr wPtr, int argc, char **argv,
 			    int flags));
 static void		WidgetDestroy _ANSI_ARGS_((ClientData clientData));
 static void		WidgetEventProc _ANSI_ARGS_((ClientData clientData,
 			    XEvent *eventPtr));
 static int		WidgetCommand _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *, int argc, CONST84 char **argv));
+			    Tcl_Interp *, int argc, char **argv));
 static void		WidgetDisplay _ANSI_ARGS_((ClientData clientData));
 static void		WidgetComputeGeometry _ANSI_ARGS_((
 			    ClientData clientData));
@@ -204,7 +202,7 @@ static void		WidgetComputeGeometry _ANSI_ARGS_((
 static void		CancelRedrawWhenIdle _ANSI_ARGS_((WidgetPtr wPtr));
 static void		CancelResizeWhenIdle _ANSI_ARGS_((WidgetPtr wPtr));
 static int		ConfigElement _ANSI_ARGS_((WidgetPtr wPtr,
-			    ListEntry *chPtr, int argc, CONST84 char ** argv,
+			    ListEntry *chPtr, int argc, char ** argv,
 			    int flags, int forced));
 static void 		RedrawRows _ANSI_ARGS_((
 			    WidgetPtr wPtr, Drawable pixmap));
@@ -217,7 +215,7 @@ static void		UpdateScrollBars _ANSI_ARGS_((WidgetPtr wPtr,
 			    int sizeChanged));
 static int		Tix_TLGetFromTo _ANSI_ARGS_((
 			    Tcl_Interp *interp, WidgetPtr wPtr,
-			    int argc, CONST84 char **argv,
+			    int argc, char **argv,
 			    ListEntry ** fromPtr_ret, ListEntry ** toPtr_ret));
 static void		Tix_TLDItemSizeChanged _ANSI_ARGS_((
 			    Tix_DItem *iPtr));
@@ -226,13 +224,13 @@ static void		MakeGeomRequest _ANSI_ARGS_((
 static int		Tix_TLGetNearest _ANSI_ARGS_((
    			    WidgetPtr wPtr, int posn[2]));
 static int		Tix_TLGetAt _ANSI_ARGS_((WidgetPtr wPtr,
-			    Tcl_Interp *interp, CONST84 char * spec, int *at));
+			    Tcl_Interp *interp, char * spec, int *at));
 static int		Tix_TLGetNeighbor _ANSI_ARGS_((
 			    WidgetPtr wPtr, Tcl_Interp *interp,
-			    int type, int argc, CONST84 char **argv));
+			    int type, int argc, char **argv));
 static int		Tix_TranslateIndex _ANSI_ARGS_((
-			    WidgetPtr wPtr, Tcl_Interp *interp, CONST84 char * string,
-			    int * index, int isInsert));
+			    WidgetPtr wPtr, Tcl_Interp *interp, char * string,
+			    int * index));
 static int		Tix_TLDeleteRange _ANSI_ARGS_((
 			    WidgetPtr wPtr, ListEntry * fromPtr,
 			    ListEntry *toPtr));
@@ -282,9 +280,9 @@ Tix_TListCmd(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
-    Tk_Window mainwin = (Tk_Window) clientData;
+    Tk_Window main = (Tk_Window) clientData;
     WidgetPtr wPtr;
     Tk_Window tkwin;
 
@@ -294,7 +292,7 @@ Tix_TListCmd(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_CreateWindowFromPath(interp, mainwin, argv[1], (char *) NULL);
+    tkwin = Tk_CreateWindowFromPath(interp, main, argv[1], (char *) NULL);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
@@ -322,7 +320,6 @@ Tix_TListCmd(clientData, interp, argc, argv)
     wPtr->backgroundGC		= None;
     wPtr->selectGC		= None;
     wPtr->anchorGC		= None;
-    wPtr->anchorGC2		= None;
     wPtr->highlightWidth	= 0;
     wPtr->highlightColorPtr	= NULL;
     wPtr->highlightGC		= None;
@@ -332,13 +329,12 @@ Tix_TListCmd(clientData, interp, argc, argv)
     wPtr->resizing 		= 0;
     wPtr->hasFocus 		= 0;
     wPtr->selectMode		= NULL;
-    wPtr->seeElemPtr 		= NULL;
-    wPtr->anchor 		= NULL;
-    wPtr->active 		= NULL;
-    wPtr->dropSite 		= NULL;
-    wPtr->dragSite 		= NULL;
-    wPtr->sizeCmd		= NULL;
-    wPtr->browseCmd		= NULL;
+    wPtr->anchor 		= 0;
+    wPtr->active 		= 0;
+    wPtr->dropSite 		= 0;
+    wPtr->dragSite 		= 0;
+    wPtr->sizeCmd		= 0;
+    wPtr->browseCmd		= 0;
     wPtr->takeFocus		= NULL;
     wPtr->orientUid		= NULL;
     wPtr->serial		= 0;
@@ -394,7 +390,7 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
     Tcl_Interp *interp;			/* Used for error reporting. */
     WidgetPtr wPtr;			/* Information about widget. */
     int argc;				/* Number of valid entries in argv. */
-    CONST84 char **argv;		/* Arguments. */
+    char **argv;			/* Arguments. */
     int flags;				/* Flags to pass to
 					 * Tk_ConfigureWidget. */
 {
@@ -476,21 +472,22 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
     }
     wPtr->selectGC = newGC;
 
-    /* The dotted anchor lines (for selected item) */
-    newGC = Tix_GetAnchorGC(wPtr->dispData.tkwin,
-            Tk_3DBorderColor(wPtr->selectBorder));
+    /* The dotted anchor lines */
+    gcValues.foreground 	= wPtr->normalFg->pixel;
+    gcValues.background 	= wPtr->normalBg->pixel;
+    gcValues.graphics_exposures = False;
+    gcValues.line_style         = LineDoubleDash;
+    gcValues.dashes 		= 2;
+    gcValues.subwindow_mode	= IncludeInferiors;
+
+    newGC = Tk_GetGC(wPtr->dispData.tkwin,
+	GCForeground|GCBackground|GCGraphicsExposures|GCLineStyle|GCDashList|
+	GCSubwindowMode, &gcValues);
     if (wPtr->anchorGC != None) {
 	Tk_FreeGC(wPtr->dispData.display, wPtr->anchorGC);
     }
     wPtr->anchorGC = newGC;
 
-    /* The dotted anchor lines (for unselected item)  */
-    newGC = Tix_GetAnchorGC(wPtr->dispData.tkwin, wPtr->normalBg);
-    if (wPtr->anchorGC2 != None) {
-	Tk_FreeGC(wPtr->dispData.display, wPtr->anchorGC2);
-    }
-    wPtr->anchorGC2 = newGC;
-
     /* The highlight border */
     gcValues.background 	= wPtr->selectFg->pixel;
     gcValues.foreground 	= wPtr->highlightColorPtr->pixel;
@@ -520,8 +517,8 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
 
     Tix_SetDefaultStyleTemplate(wPtr->dispData.tkwin, &stTmpl);
 
-    /*
-     * Probably the -font or -width or -height options have changed. Let's
+
+    /* Probably the -font or -width or -height options have changed. Let's
      * make geometry request
      */
     MakeGeomRequest(wPtr);
@@ -554,7 +551,7 @@ WidgetCommand(clientData, interp, argc, argv)
     ClientData clientData;		/* Information about the widget. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
-    CONST84 char **argv;		/* Argument strings. */
+    char **argv;			/* Argument strings. */
 {
     int code;
 
@@ -699,16 +696,13 @@ WidgetDestroy(clientData)
     if (wPtr->anchorGC != None) {
 	Tk_FreeGC(wPtr->dispData.display, wPtr->anchorGC);
     }
-    if (wPtr->anchorGC2 != None) {
-	Tk_FreeGC(wPtr->dispData.display, wPtr->anchorGC2);
-    }
     if (wPtr->highlightGC != None) {
 	Tk_FreeGC(wPtr->dispData.display, wPtr->highlightGC);
     }
 
     if (wPtr->entList.numItems > 0) {
 	ListEntry * fromPtr=NULL, *toPtr=NULL;
-	CONST84 char * argv[2];
+	char * argv[2];
 	argv[0] = "0";
 	argv[1] = "end";
 
@@ -800,7 +794,6 @@ WidgetComputeGeometry(clientData)
 
     RedrawWhenIdle(wPtr);
 }
-
 static void
 MakeGeomRequest(wPtr)
     WidgetPtr wPtr;
@@ -843,9 +836,6 @@ static void
 ResizeWhenIdle(wPtr)
     WidgetPtr wPtr;
 {
-    if (wPtr->redrawing) {
-	CancelRedrawWhenIdle(wPtr);
-    }
     if (!wPtr->resizing) {
 	wPtr->resizing = 1;
 	Tk_DoWhenIdle(WidgetComputeGeometry, (ClientData)wPtr);
@@ -893,12 +883,6 @@ static void
 RedrawWhenIdle(wPtr)
     WidgetPtr wPtr;
 {
-    if (wPtr->resizing) {
-	/*
-	 * Resize will eventually call redraw.
-	 */
-	return;
-    }
     if (!wPtr->redrawing && Tk_IsMapped(wPtr->dispData.tkwin)) {
 	wPtr->redrawing = 1;
 	Tk_DoWhenIdle(WidgetDisplay, (ClientData)wPtr);
@@ -942,8 +926,6 @@ WidgetDisplay(clientData)
     Pixmap pixmap;
     Tk_Window tkwin = wPtr->dispData.tkwin;
     int winH, winW;
-    int hl = wPtr->highlightWidth;
-    int bd = wPtr->borderWidth;
 
     wPtr->redrawing = 0;		/* clear the redraw flag */
     wPtr->serial ++;
@@ -953,10 +935,10 @@ WidgetDisplay(clientData)
 
     /* Fill the background */
     XFillRectangle(wPtr->dispData.display, pixmap, wPtr->backgroundGC,
-	0, 0, (unsigned) Tk_Width(tkwin), (unsigned) Tk_Height(tkwin));
+	0, 0, Tk_Width(tkwin), Tk_Height(tkwin));
 
-    winW = Tk_Width(tkwin)  - 2*hl - 2*bd;
-    winH = Tk_Height(tkwin) - 2*hl - 2*bd;
+    winW = Tk_Width(tkwin)  - 2*wPtr->highlightWidth - 2*wPtr->borderWidth;
+    winH = Tk_Height(tkwin) - 2*wPtr->highlightWidth - 2*wPtr->borderWidth;
 
     if (winW > 0 && winH > 0) {
 	RedrawRows(wPtr, pixmap);
@@ -964,11 +946,13 @@ WidgetDisplay(clientData)
 
     /* Draw the border */
     Tk_Draw3DRectangle(wPtr->dispData.tkwin, pixmap, wPtr->border,
-	    hl, hl, Tk_Width(tkwin)  - 2*hl, Tk_Height(tkwin) - 2*hl, bd,
-	    wPtr->relief);
+	wPtr->highlightWidth, wPtr->highlightWidth,
+	Tk_Width(tkwin)  - 2*wPtr->highlightWidth, 
+	Tk_Height(tkwin) - 2*wPtr->highlightWidth, wPtr->borderWidth,
+	wPtr->relief);
   
     /* Draw the highlight */
-    if (hl > 0) {
+    if (wPtr->highlightWidth > 0) {
 	GC gc;
 
 	if (wPtr->hasFocus) {
@@ -977,7 +961,7 @@ WidgetDisplay(clientData)
 	    gc = Tk_3DBorderGC(wPtr->dispData.tkwin, wPtr->border, 
 		TK_3D_FLAT_GC);
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, hl, pixmap);
+	Tk_DrawFocusHighlight(tkwin, gc, wPtr->highlightWidth, pixmap);
     }
 
     /*
@@ -985,8 +969,7 @@ WidgetDisplay(clientData)
      * then delete the pixmap.
      */
     XCopyArea(wPtr->dispData.display, pixmap, Tk_WindowId(tkwin),
-	    wPtr->backgroundGC, 0, 0,
-	    (unsigned) Tk_Width(tkwin), (unsigned) Tk_Height(tkwin), 0, 0);
+	wPtr->backgroundGC, 0, 0, Tk_Width(tkwin), Tk_Height(tkwin), 0, 0);
     Tk_FreePixmap(wPtr->dispData.display, pixmap);
 
 }
@@ -1061,28 +1044,6 @@ FreeEntry(wPtr, chPtr)
     WidgetPtr wPtr;
     ListEntry * chPtr;
 {
-
-    if (wPtr->seeElemPtr == chPtr) {
-	/*
-	 * This is the element that should be visible the next time
-	 * we draw the window. Adjust the "to see element" to an element
-	 * that is close to it.
-	 */
-	if (chPtr->next != NULL) {
-	    wPtr->seeElemPtr = chPtr->next;
-	} else {
-	    ListEntry *p;
-
-	    wPtr->seeElemPtr = NULL;
-	    for (p=(ListEntry*)wPtr->entList.head; p; p=p->next) {
-		if (p->next == chPtr) {
-		    wPtr->seeElemPtr = p;
-		    break;
-		}
-	    }
-	}
-    }
-
     if (wPtr->anchor == chPtr) {
 	wPtr->anchor = NULL;
     }
@@ -1121,12 +1082,12 @@ Tix_TLInsert(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     ListEntry * chPtr = NULL;
     char buff[40];
-    CONST84 char * ditemType;
+    char * ditemType;
     int at;
     int added = 0;
     int code = TCL_OK;
@@ -1138,7 +1099,7 @@ Tix_TLInsert(clientData, interp, argc, argv)
      */
 
     /* (1.1) Find out where */
-    if (Tix_TranslateIndex(wPtr, interp, argv[0], &at, 1) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, argv[0], &at) != TCL_OK) {
 	code = TCL_ERROR; goto done;
     }
 
@@ -1251,13 +1212,13 @@ Tix_TLIndex(clientData, interp, argc, argv)
     ClientData clientData;	/* TList widget record. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     int index;
     char buff[100];
 
-    if (Tix_TranslateIndex(wPtr, interp, argv[0], &index, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, argv[0], &index) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -1275,7 +1236,7 @@ Tix_TLInfo(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     size_t len = strlen(argv[0]);
@@ -1314,14 +1275,6 @@ Tix_TLInfo(clientData, interp, argc, argv)
 	}
 	return TCL_OK;
     }
-    else if (strncmp(argv[0], "size", len)==0) {
-	char buff[100];
-
-	sprintf(buff, "%d", wPtr->entList.numItems);
-	Tcl_AppendResult(interp, buff, NULL);
-
-	return TCL_OK;
-    }
     else if (strncmp(argv[0], "up", len)==0) {
 	return Tix_TLGetNeighbor(wPtr, interp, TIX_UP,    argc-1, argv+1);
     }
@@ -1334,13 +1287,11 @@ Tix_TLInfo(clientData, interp, argc, argv)
 }
 
 static int
-Tix_TranslateIndex(wPtr, interp, string, index, isInsert)
+Tix_TranslateIndex(wPtr, interp, string, index)
     WidgetPtr wPtr;		/* TList widget record. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    CONST84 char * string;	/* String representation of the index. */
-    int * index;		/* Returns the index value(0 = 1st element).*/
-    int isInsert;		/* Is this function called by an "insert"
-				 * operation? */
+    char * string;		/* String representation of the index. */
+    int * index;		/* Returns the index value (0 = 1st element).*/
 {
     if (strcmp(string, "end") == 0) {
 	*index = wPtr->entList.numItems;
@@ -1356,31 +1307,12 @@ Tix_TranslateIndex(wPtr, interp, string, index, isInsert)
 	}
     }
 
-
-    /*
-     * The meaning of "end" means:
-     *     isInsert:wPtr->entList.numItems
-     *    !isInsert:wPtr->entList.numItems-1;
-     */
-
-    if (isInsert) {
-	if (*index > wPtr->entList.numItems) {
-	    /*
-	     * By default add it to the end, just to follow what TK
-	     * does for the Listbox widget
-	     */
-	    *index = wPtr->entList.numItems;
-	}
-    } else {
-	if (*index >= wPtr->entList.numItems) {
-	    /*
-	     * By default add it to the end, just to follow what TK
-	     * does for the Listbox widget
-	     */
-	    *index = wPtr->entList.numItems - 1;
-	}
+    if (*index > wPtr->entList.numItems) {
+	/* By default add it to the end, just to follow what TK
+	 * does for the Listbox widget
+	 */
+	*index = wPtr->entList.numItems;
     }
-
     if (*index < 0) {
 	*index = 0;
     }
@@ -1393,10 +1325,10 @@ static int Tix_TLGetNeighbor(wPtr, interp, type, argc, argv)
     Tcl_Interp *interp;		/* Current interpreter. */
     int type;
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     int index;
-    int dst = 0; /* lint */
+    int dst;
     int xStep, yStep;
     int numPerRow;
     char buff[100];
@@ -1405,7 +1337,7 @@ static int Tix_TLGetNeighbor(wPtr, interp, type, argc, argv)
 	Tix_ArgcError(interp, argc+3, argv-3, 3, "index");
     }
 
-    if (Tix_TranslateIndex(wPtr, interp, argv[0], &index, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, argv[0], &index) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -1462,7 +1394,7 @@ Tix_TLCGet(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
 
@@ -1479,7 +1411,7 @@ Tix_TLConfig(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
 
@@ -1504,7 +1436,7 @@ Tix_TLGeometryInfo(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     int qSize[2];
@@ -1547,7 +1479,7 @@ Tix_TLDelete(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     ListEntry * fromPtr, *toPtr;
@@ -1559,7 +1491,8 @@ Tix_TLDelete(clientData, interp, argc, argv)
 	goto done;
     }
 
-    if (Tix_TLGetFromTo(interp, wPtr, argc, argv, &fromPtr, &toPtr)!= TCL_OK) {
+    if (Tix_TLGetFromTo(interp, wPtr, argc, argv, &fromPtr, &toPtr)
+	!= TCL_OK) {
 	code = TCL_ERROR;
 	goto done;
     }
@@ -1616,7 +1549,7 @@ Tix_TLEntryCget(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     ListEntry * chPtr, * dummy;
@@ -1645,7 +1578,7 @@ Tix_TLEntryConfig(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     ListEntry * chPtr, * dummy;
@@ -1682,7 +1615,7 @@ Tix_TLNearest(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     int posn[2];
@@ -1709,7 +1642,7 @@ Tix_TLNearest(clientData, interp, argc, argv)
 static int Tix_TLGetAt(wPtr, interp, spec, at)
     WidgetPtr wPtr;
     Tcl_Interp *interp;
-    CONST84 char * spec;
+    char * spec;
     int *at;
 {
     int posn[2];
@@ -1719,7 +1652,7 @@ static int Tix_TLGetAt(wPtr, interp, spec, at)
 	return TCL_ERROR;
     }
 
-    p = (char *) spec+1;
+    p = spec+1;
     posn[0] = strtol(p, &end, 0);
     if ((end == p) || (*end != ',')) {
 	return TCL_ERROR;
@@ -1802,7 +1735,7 @@ Tix_TLGetFromTo(interp, wPtr, argc, argv, fromPtr_ret, toPtr_ret)
     Tcl_Interp *interp;
     WidgetPtr wPtr;
     int argc;
-    CONST84 char **argv;
+    char **argv;
     ListEntry ** fromPtr_ret;
     ListEntry ** toPtr_ret;
 {
@@ -1814,11 +1747,11 @@ Tix_TLGetFromTo(interp, wPtr, argc, argv, fromPtr_ret, toPtr_ret)
     ListEntry * toPtr;
     int from, to, tmp;
 
-    if (Tix_TranslateIndex(wPtr, interp, argv[0], &from, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, argv[0], &from) != TCL_OK) {
 	return TCL_ERROR;
     }
     if (argc == 2) {
-	if (Tix_TranslateIndex(wPtr, interp, argv[1], &to, 0) != TCL_OK) {
+	if (Tix_TranslateIndex(wPtr, interp, argv[1], &to) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     } else {
@@ -1868,7 +1801,7 @@ Tix_TLSelection(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     size_t len = strlen(argv[0]);
@@ -1971,58 +1904,22 @@ Tix_TLSelection(clientData, interp, argc, argv)
     return code;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * Tix_TLSee --
- *
- *      Implements the "see" widget subcommand -- make sure that the
- *      given entry is visible as much as possible.
- *
- *      We don't scroll the widget immediately in this function.
- *      Instead, we set the wPtr->seeElemPtr and the scrolling will be
- *      set when the widget is redrawn. This make sure that the "see"
- *      command works even before the widget's contents has been laid
- *      out.
- *
- * Results:
- *      A standard Tcl result.
- *
- * Side effects:
- *      Widget is scheduled to be redrawn.
- *
+/*----------------------------------------------------------------------
+ * "see" command
  *----------------------------------------------------------------------
  */
-
 static int
 Tix_TLSee(clientData, interp, argc, argv)
-    ClientData clientData;      /* The widget data struct */
+    ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
-    WidgetPtr wPtr = (WidgetPtr) clientData;
-    ListEntry * chPtr, * dummy;
-
-    if (argc == 1) {
-	if (Tix_TLGetFromTo(interp, wPtr, 1, argv, &chPtr, &dummy) != TCL_OK) {
-	    return TCL_ERROR;
-	}
-	if (chPtr != NULL) {
-	    wPtr->seeElemPtr = chPtr;
-	    RedrawWhenIdle(wPtr);
-	}
-    } else {
-	Tcl_AppendResult(interp, "wrong # of arguments, must be: ",
-		Tk_PathName(wPtr->dispData.tkwin), " ", argv[-1],
-		" index", NULL);
-    }
-
     return TCL_OK;
 }
 
 /*----------------------------------------------------------------------
- * "anchor", "dragsite" and "dropsite" sub commands --
+ * "anchor", "dragsite" and "dropsire" sub commands --
  *
  *	Set/remove the anchor element
  *----------------------------------------------------------------------
@@ -2032,7 +1929,7 @@ Tix_TLSetSite(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     int changed = 0;
     WidgetPtr wPtr = (WidgetPtr) clientData;
@@ -2105,7 +2002,7 @@ Tix_TLView(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     int axis;
@@ -2150,7 +2047,7 @@ ConfigElement(wPtr, chPtr, argc, argv, flags, forced)
     WidgetPtr wPtr;
     ListEntry *chPtr;
     int argc;
-    CONST84 char ** argv;
+    char ** argv;
     int flags;
     int forced;
 {
@@ -2183,8 +2080,7 @@ Realloc(wPtr, new_size)
     if (new_size == wPtr->numRowAllocd) {
 	return;
     }
-    wPtr->rows = (ListRow*)ckrealloc( (char *)wPtr->rows,
-                                      sizeof(ListRow)*new_size);
+    wPtr->rows = (ListRow*)ckrealloc(wPtr->rows, sizeof(ListRow)*new_size);
     wPtr->numRowAllocd = new_size;
 }
 
@@ -2325,10 +2221,9 @@ static void ResizeRows(wPtr, winW, winH)
 /*----------------------------------------------------------------------
  * RedrawRows --
  *
- *	Redraw the rows, according to the "offset" in both directions
+ *	Redraw the rows, according to the "offset: in both directions
  *----------------------------------------------------------------------
  */
-
 static void
 RedrawRows(wPtr, pixmap)
     WidgetPtr wPtr;
@@ -2339,7 +2234,6 @@ RedrawRows(wPtr, pixmap)
     ListEntry * chPtr;
     int i, j;
     int total;
-    int windowSize;
 
     if (wPtr->entList.numItems == 0) {
 	return;
@@ -2348,66 +2242,14 @@ RedrawRows(wPtr, pixmap)
     if (wPtr->isVertical) {
 	i = 0;		/* Column major, 0,0 -> 0,1 -> 0,2 ... -> 1,0 */
 	j = 1;
-	windowSize = Tk_Width(wPtr->dispData.tkwin);
     } else {
 	i = 1;		/* Row major, 0,0 -> 1,0 -> 2,0 ... -> 0,1 */
 	j = 0;
-	windowSize = Tk_Height(wPtr->dispData.tkwin);
     }
 
     p[i] = wPtr->highlightWidth + wPtr->borderWidth;
-    windowSize -= 2*p[i];
 
-    if (windowSize < 1) {
-	windowSize = 1;
-    }
-
-    if (wPtr->seeElemPtr != NULL) {
-	/*
-	 * Adjust the scrolling so that the given the entry is visible in
-         * its entirty (as much as allowed by the height of the TList)
-	 */
-
-	int start = 0;		/* y1 position of the element to see. */
-	int size = 0;		/* height of the element to see. */
-	int old = wPtr->scrollInfo[i].offset;
-
-	for (r=0, n=0, chPtr=(ListEntry*)wPtr->entList.head; chPtr;
-		chPtr=chPtr->next) {
-	    if (chPtr == wPtr->seeElemPtr) {
-		size = chPtr->size[i];
-		break;
-	    }
-            n++;
-	    if (n == wPtr->rows[r].numEnt) {
-		n=0;
-		r++;
-		start += wPtr->rows[r].size[i];
-	    }
-	}
-
-	if (wPtr->scrollInfo[i].offset + windowSize < start + size) {
-            /*
-             * Bottom of the entry is beneath the bottom edge
-             */
-	    wPtr->scrollInfo[i].offset = start + size - windowSize;
-	}
-	if (wPtr->scrollInfo[i].offset > start) {
-            /*
-             * Top of the entry is above the top edge
-             */
-	    wPtr->scrollInfo[i].offset = start;
-	}
-	if (wPtr->scrollInfo[i].offset != old) {
-	    UpdateScrollBars(wPtr, 0);
-	}
-	wPtr->seeElemPtr = NULL;
-    }
-
-    /*
-     * Search for the first row from the top (or first column from the left)
-     * that is (possibly partially) visible
-     */
+    /* Search for a row that is (possibly partially) visible*/
     total=0; r=0;
     if (wPtr->scrollInfo[i].offset != 0) {
 	for (; r<wPtr->numRow; r++) {
@@ -2453,7 +2295,7 @@ RedrawRows(wPtr, pixmap)
 
 	/* Redraw all the visible columns in this row */
 	for (; n<wPtr->rows[r].numEnt; n++, chPtr = chPtr->next) {
-	    int flags = TIX_DITEM_NORMAL_FG | TIX_DITEM_NORMAL_BG;
+	    int flags = TIX_DITEM_NORMAL_FG;
 	    int W, H;
 
 	    if (chPtr->selected) {
@@ -2469,25 +2311,13 @@ RedrawRows(wPtr, pixmap)
 		W = chPtr->iPtr->base.size[0];
 	    }
 
-            if (chPtr == wPtr->anchor && wPtr->hasFocus) {
-                flags |= TIX_DITEM_ANCHOR;
-            }
+	    Tix_DItemDisplay(pixmap, None, chPtr->iPtr, p[0], p[1], W, H,
+		flags);
 
-	    Tix_DItemDisplay(pixmap, chPtr->iPtr, p[0], p[1], W, H,
-		0, 0, flags);
-
-#if 0
-	    if (chPtr == wPtr->anchor && wPtr->hasFocus) {
-                GC ancGC;
-                if (chPtr->selected) {
-                    ancGC = wPtr->anchorGC;
-                } else {
-                    ancGC = wPtr->anchorGC2;
-                }
+	    if (chPtr == wPtr->anchor) {
 		Tix_DrawAnchorLines(Tk_Display(wPtr->dispData.tkwin), pixmap,
-	                ancGC, p[0], p[1], W, H);
+	            wPtr->anchorGC, p[0], p[1], W-1, H-1);
 	    }
-#endif
 	    p[j] += wPtr->maxSize[j];
 	}
 
