@@ -11,16 +11,12 @@
  *
  */
 
-#include <tclInt.h>
-#include <tk.h>
-#include <tkPort.h>
-#include <tix.h>
-
 /*
- * Declarations for various library procedures and variables (don't want
- * to include tkInt.h or tkConfig.h here, because people might copy this
- * file out of the Tk source directory to make their own modified versions).
+ * tclInt.h is needed for the va_list declaration.
  */
+#include <tclInt.h>
+#include <tixPort.h>
+#include <tixInt.h>
 
 /*
  * Forward declarations for procedures defined later in this file:
@@ -30,23 +26,110 @@ static void		Prompt _ANSI_ARGS_((Tcl_Interp *interp, int partial));
 static void		StdinProc _ANSI_ARGS_((ClientData clientData,
 			    int mask));
 
+static int		ReliefParseProc _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, Tk_Window tkwin, char *value,
+			    char *widRec, int offset));
+static char *		ReliefPrintProc _ANSI_ARGS_((ClientData clientData,
+			    Tk_Window tkwin, char *widRec, int offset,
+			    Tix_FreeProc **freeProcPtr));
 /*
- * Some ugly but necessary global vars used in this file
+ * Global vars used in this file
  */
 static Tcl_DString command;	/* Used to assemble lines of terminal input
 				 * into Tcl commands. */
 
 
-/* Tix_HandleSubCmds --
+#define WRONG_ARGC 1
+#define NO_MATCH   2
+
+
+/*----------------------------------------------------------------------
+ * TixSaveInterpState --
  *
- *	This function makes it easier to write major-minor style TCL commands.
- * It matches the minor command (sub-command) names with names defined in
- * the cmdInfo structure and call the appropriate sub-command functions for
- * you. This function will automatically generate error messages when
- * the user calls an invalid sub-command or calls a sub-command with
- * incorrect number of arguments.
+ *	Save the current application-visible state of the interpreter.
+ *	This can later be restored by the TixSaveInterpState() function.
+ *	These two functions are useful if you want to evaluate a Tcl
+ *	command, which may cause errors, inside a command function.
  *
+ *	Each TixSaveInterpState() call much be matched by one
+ *	TixRestoreInterpState() call with the same statePtr. statePtr
+ *	should be allocated by the calling function, usually
+ *	as a variable on the stack.
+ *----------------------------------------------------------------------
  */
+
+void
+TixSaveInterpState(interp, statePtr)
+    Tcl_Interp * interp;
+    TixInterpState * statePtr;
+{
+    char * p;
+    if (interp->result) {
+	statePtr->result = (char*)strdup(interp->result);
+    } else {
+	statePtr->result = NULL;
+    }
+
+    p = Tcl_GetVar2(interp, "errorInfo", NULL, TCL_GLOBAL_ONLY);
+    if (p) {
+	statePtr->errorInfo = (char*)strdup(p);
+    } else {
+	statePtr->errorInfo = NULL;
+    }
+
+    p = Tcl_GetVar2(interp, "errorCode", NULL, TCL_GLOBAL_ONLY);
+    if (p) {
+	statePtr->errorCode = (char*)strdup(p);
+    } else {
+	statePtr->errorCode = NULL;
+    }
+}
+
+/*----------------------------------------------------------------------
+ * TixRestoreInterpState --
+ *
+ *	See TixSaveInterpState above.
+ *----------------------------------------------------------------------
+ */
+
+void
+TixRestoreInterpState(interp, statePtr)
+    Tcl_Interp * interp;
+    TixInterpState * statePtr;
+{
+    if (statePtr->result) {
+	Tcl_SetResult(interp, statePtr->result, TCL_DYNAMIC);
+    }
+    if (statePtr->errorInfo) {
+	Tcl_SetVar2(interp, "errorInfo", NULL, statePtr->errorInfo,
+		TCL_GLOBAL_ONLY);
+	ckfree((char*)statePtr->errorInfo);
+    } else {
+	Tcl_UnsetVar2(interp, "errorInfo", NULL, TCL_GLOBAL_ONLY);
+    }
+    if (statePtr->errorCode) {
+	Tcl_SetVar2(interp, "errorCode", NULL, statePtr->errorCode,
+		TCL_GLOBAL_ONLY);
+	ckfree((char*)statePtr->errorCode);
+    } else {
+	Tcl_UnsetVar2(interp, "errorCode", NULL, TCL_GLOBAL_ONLY);
+    }
+}
+
+/*----------------------------------------------------------------------
+ * Tix_HandleSubCmds --
+ *
+ *	This function makes it easier to write major-minor style TCL
+ *	commands.  It matches the minor command (sub-command) names
+ *	with names defined in the cmdInfo structure and call the
+ *	appropriate sub-command functions for you. This function will
+ *	automatically generate error messages when the user calls an
+ *	invalid sub-command or calls a sub-command with incorrect
+ *	number of arguments.
+ *
+ *----------------------------------------------------------------------
+ */
+
 int Tix_HandleSubCmds(cmdInfo, subCmdInfo, clientData, interp, argc, argv)
     Tix_CmdInfo * cmdInfo;
     Tix_SubCmdInfo * subCmdInfo;
@@ -56,7 +139,6 @@ int Tix_HandleSubCmds(cmdInfo, subCmdInfo, clientData, interp, argc, argv)
     int argc;			/* Number of arguments. */
     char **argv;		/* Argument strings. */
 {
-    enum {WRONG_ARGC, NO_MATCH};
 
     int i;
     int len;
@@ -123,7 +205,7 @@ int Tix_HandleSubCmds(cmdInfo, subCmdInfo, clientData, interp, argc, argv)
 	 * got a match but incorrect number of arguments
 	 */
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
-	    argv[0], " ", argv[1], " ", s->info, (char *) NULL);
+	    argv[0], " ", argv[1], " ", s->info, "\"", (char *) NULL);
     } else {
 	int max;
 
@@ -166,13 +248,14 @@ int Tix_HandleSubCmds(cmdInfo, subCmdInfo, clientData, interp, argc, argv)
     }
     return TCL_ERROR;
 }
-
-/*
- * Tix_Exit
+
+/*----------------------------------------------------------------------
+ * Tix_Exit --
  *
- * Call the "exit" tcl command so that things can be cleaned up before
- * calling the unix exit(2);
+ *	Call the "exit" tcl command so that things can be cleaned up
+ *	before calling the unix exit(2);
  *
+ *----------------------------------------------------------------------
  */
 void Tix_Exit(interp, code)
     Tcl_Interp* interp;
@@ -221,21 +304,20 @@ int Tix_ShellInputCmd(clientData, interp, argc, argv)
     char **argv;		/* Argument strings. */
 {
 
-#ifndef _Windows
+#ifdef _WINDOWS
+    /*
+     * This command is only defined if we are running in the txwish.exe
+     * program, not in a Tix.DLL
+     */
+    Tcl_Eval(interp, "tixConsoleInit");
+    Tcl_ResetResult(interp);
+#else
     Tix_OpenStdin(interp);
+#endif
 
     return TCL_OK;
-#else
-#ifdef NO_DLL
-    return TkConsoleInit(interp);
-#else
-    return TCL_OK;
-#endif
-#endif
 }
-
-
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -266,7 +348,7 @@ StdinProc(clientData, mask)
     char input[BUFFER_SIZE+1];
     static int gotPartial = 0;
     char *cmd;
-    int code, count;
+    int count;
     Tcl_Interp * interp = (Tcl_Interp*)clientData;
 
     count = read(fileno(stdin), input, BUFFER_SIZE);
@@ -311,7 +393,7 @@ StdinProc(clientData, mask)
      */
 
     Tk_CreateFileHandler(0, 0, StdinProc, (ClientData) interp);
-    code = Tcl_RecordAndEval(interp, cmd, 0);
+    Tcl_RecordAndEval(interp, cmd, 0);
     Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) interp);
     Tcl_DStringFree(&command);
     if (interp->result != 0 && isatty(0)) {
@@ -396,6 +478,7 @@ Prompt(interp, partial)
 /* Some compilers can't handle multi-line character strings very well ...
  * So I just using this big lump of mess here.
  */
+
 static char _format[] = "lappend auto_path $%s \nif [file exists $%s/%s] {\nsource $%s/%s\n} else {\nset msg \"\ncan't find $%s/%s;\\nperhaps you \"\nappend msg \"need to install %s\\nor set your %s \"\nappend msg \"environment variable?\"\nerror $msg\n}";
 
 int
@@ -407,16 +490,21 @@ Tix_LoadTclLibrary(interp, envName, tclName, initFile, defDir, appName)
     char *defDir;
     char *appName;
 {
-    char * libDir, *initCmd;	/* should be big enough */
+    char * libDir, *initCmd;
     size_t size;
     int code;
     char *format;
     format = _format;
 
-    if (!(libDir = getenv(envName))) {
-	libDir = defDir; 
+    libDir = getenv(envName);
+    if (libDir == NULL) {
+	libDir = defDir;
     }
 
+    /*
+     * This size should be big enough.
+     */
+
     size = strlen(format) + strlen(tclName)*4 + strlen(initFile)*3
 	+ strlen(appName) + strlen(envName) + 100;
     initCmd = ckalloc(sizeof(char) * size);
@@ -435,13 +523,13 @@ Tix_LoadTclLibrary(interp, envName, tclName, initFile, defDir, appName)
     ckfree(initCmd);
     return code;
 }
-
+
 /*----------------------------------------------------------------------
  * Tix_CreateCommands --
  *
  *
  *	Creates a list of commands stored in the array "commands"
- *
+ *----------------------------------------------------------------------
  */
 
 void Tix_CreateCommands(interp, commands, clientData, deleteProc)
@@ -462,8 +550,9 @@ void Tix_CreateCommands(interp, commands, clientData, deleteProc)
  * Tix_DrawAnchorLines --
  *
  * 	Draw dotted anchor lines around anchor elements
- *
+ *----------------------------------------------------------------------
  */
+
 void Tix_DrawAnchorLines(display, drawable, gc, x, y, w, h)
     Display *display;
     Drawable drawable;
@@ -473,38 +562,17 @@ void Tix_DrawAnchorLines(display, drawable, gc, x, y, w, h)
     int w;
     int h;
 {
-    XPoint points[4];
-
-    if (w < 1) {
-	w = 1;
-    }
-    if (h < 1) {
-	h = 1;
-    }
-
-    XDrawRectangle(display, drawable, gc, x, y, w, h);
-
-    /* Draw these points so that the corners will not be rounded */
-    points[0].x = x;
-    points[0].y = y;
-    points[1].x = x+w;
-    points[1].y = y;
-    points[2].x = x;
-    points[2].y = y+h;
-    points[3].x = x+w;
-    points[3].y = y+h;
-    XDrawPoints(display, drawable, gc,
-	points, 4, CoordModeOrigin);
+    TixpDrawAnchorLines(display, drawable, gc, x, y, w, h);
 }
 
 /*----------------------------------------------------------------------
  * Tix_CreateSubWindow --
  *
- *
  *	Creates a subwindow for a widget (usually used to draw headers,
  *	e.g, HList and Grid widgets)
- *
+ *----------------------------------------------------------------------
  */
+
 Tk_Window
 Tix_CreateSubWindow(interp, tkwin, subPath)
     Tcl_Interp * interp;
@@ -538,8 +606,9 @@ Tix_CreateSubWindow(interp, tkwin, subPath)
  *	of this function has two choices: (1) draw to the window directly
  *	(which may lead to flashing on the screen) or (2) try to allocate
  *	smaller pixmaps.
- *
+ *----------------------------------------------------------------------
  */
+
 static int
 ErrorProc(clientData, errorEventPtr)
     ClientData clientData;
@@ -567,12 +636,14 @@ Drawable Tix_GetRenderBuffer(display, windowId, width, height, depth)
 	-1, -1, (Tk_ErrorProc *) ErrorProc, (ClientData) &badAlloc);
     pixmap = Tk_GetPixmap(display, windowId, width, height, depth);
 
-    /* This XSync call is necessary because X may delay the delivery of the
+#ifndef _WINDOWS
+    /*
+     * This XSync call is necessary because X may delay the delivery of the
      * error message. This will make our graphics a bit slower, though,
      * especially over slow lines
      */
     XSync(display, 0);
-
+#endif
     /* If ErrorProc() is eevr called, it is called before XSync returns */
 
     Tk_DeleteErrorHandler(handler);
@@ -664,6 +735,95 @@ Tix_GlobalVarEval(iPtr, p, va_alist)
     return result;
 #endif
 }
+
+/*----------------------------------------------------------------------
+ * TixGetHashTable --
+ *
+ *	This functions makes it possible to keep one hash table per
+ *	interpreter. This way, Tix classes can be used in multiple
+ *	interpreters.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#ifdef TK_4_1_OR_LATER
+
+static void		DeleteHashTableProc _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp * interp));
+static void
+DeleteHashTableProc(clientData, interp)
+    ClientData clientData;
+    Tcl_Interp * interp;
+{
+    Tcl_HashTable * htPtr = (Tcl_HashTable *)clientData;
+    Tcl_HashSearch hashSearch;
+    Tcl_HashEntry * hashPtr;
+
+    for (hashPtr = Tcl_FirstHashEntry(htPtr, &hashSearch);
+	    hashPtr;
+	    hashPtr = Tcl_NextHashEntry(&hashSearch)) {
+	Tcl_DeleteHashEntry(hashPtr);
+    }
+
+    Tcl_DeleteHashTable(htPtr);
+    ckfree((char*)htPtr);
+}
+
+Tcl_HashTable *
+TixGetHashTable(interp, name, deleteProc)
+    Tcl_Interp * interp;
+    char * name;
+    Tcl_InterpDeleteProc *deleteProc;
+{
+    Tcl_HashTable * htPtr;
+
+    htPtr = (Tcl_HashTable*)Tcl_GetAssocData(interp, name, NULL);
+    if (htPtr == NULL) {
+	htPtr = (Tcl_HashTable *)ckalloc(sizeof(Tcl_HashTable));
+	Tcl_InitHashTable(htPtr, TCL_STRING_KEYS);
+	Tcl_SetAssocData(interp, name, NULL, (ClientData)htPtr);
+	if (deleteProc) {
+	    Tcl_CallWhenDeleted(interp, deleteProc, (ClientData)htPtr);
+	} else {
+	    Tcl_CallWhenDeleted(interp, DeleteHashTableProc,
+		    (ClientData)htPtr);
+	}
+    }
+
+    return htPtr;
+}
+
+#else
+
+Tcl_HashTable *
+TixGetHashTable(interp, name)
+    Tcl_Interp * interp;	/* Current interpreter. */
+    char * name;		/* Textual name of the hash table. */
+{
+    static int inited = 0;
+    static Tcl_HashTable classTable;
+    static Tcl_HashTable methodTable;
+    static Tcl_HashTable specTable;
+
+    if (!inited) {
+	Tcl_InitHashTable(&classTable, TCL_STRING_KEYS);
+	Tcl_InitHashTable(&methodTable, TCL_STRING_KEYS);
+	Tcl_InitHashTable(&specTable, TCL_STRING_KEYS);
+	inited = 1;
+    }
+
+    if (strcmp(name, "tixClassTab") == 0) {
+	return &classTable;
+    } else if (strcmp(name, "tixSpecTab") == 0) {
+	return &specTable;
+    } else if (strcmp(name, "tixMethodTab") == 0) {
+	return &methodTable;
+    } else {
+	panic("Unknown hash table %s", name);
+    }
+}
+#endif
+
 /*----------------------------------------------------------------------
  *
  *		 The Tix Customed Config Options
@@ -708,7 +868,7 @@ static int ReliefParseProc(clientData, interp, tkwin, value, widRec,offset)
 	    goto error;
 	}
     } else {
-	value == "";
+	value = "";
 	goto error;
     }
 
@@ -757,24 +917,128 @@ Tk_CustomOption tixConfigRelief = {
     ReliefParseProc, ReliefPrintProc, 0,
 };
 
-void Tix_SetRcFileName(interp, appName)
+/* Tix_SetRcFileName --
+ *
+ *	Sets a user-specific startup file in a way that's compatible with
+ *	different versions of Tclsh
+ */
+void Tix_SetRcFileName(interp, rcFileName)
     Tcl_Interp * interp;
-    char * appName;
+    char * rcFileName;
 {
-    char *buff = ckalloc(strlen(appName) + 100);
-
-    sprintf(buff, "~/.%src");
-
-#if ((TCL_MAJOR_VERSION > 7) || ((TCL_MAJOR_VERSION == 7) && (TCL_MINOR_VERSION >= 5)))
-    /* Starting from TCL 7.5, the symbol tcl_rcFileName is no longer
+#ifdef TCL_7_5_OR_LATER
+    /*
+     * Starting from TCL 7.5, the symbol tcl_rcFileName is no longer
      * exported by libtcl.a. Instead, this variable must be set using
      * a TCL global variable
      */
-    Tcl_SetVar(interp, "tcl_rcFileName", buff, TCL_GLOBAL_ONLY);
-    ckfree(buff);
+    Tcl_SetVar(interp, "tcl_rcFileName", rcFileName, TCL_GLOBAL_ONLY);
 #else
-    tcl_RcFileName = buff;
+    tcl_RcFileName = rcFileName;
 #endif
-
-
 }
+
+#if (TK_MAJOR_VERSION > 4)
+
+/*
+ * The TkComputeTextGeometry function is no longer supported in Tk 8.0+
+ */
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TixComputeTextGeometry --
+ *
+ *	This procedure computes the amount of screen space needed to
+ *	display a multi-line string of text.
+ *
+ * Results:
+ *	There is no return value.  The dimensions of the screen area
+ *	needed to display the text are returned in *widthPtr, and *heightPtr.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TixComputeTextGeometry(font, string, numChars, wrapLength,
+	widthPtr, heightPtr)
+    TixFont font;		/* Font that will be used to display text. */
+    char *string;		/* String whose dimensions are to be
+				 * computed. */
+    int numChars;		/* Number of characters to consider from
+				 * string. */
+    int wrapLength;		/* Longest permissible line length, in
+				 * pixels.  <= 0 means no automatic wrapping:
+				 * just let lines get as long as needed. */
+    int *widthPtr;		/* Store width of string here. */
+    int *heightPtr;		/* Store height of string here. */
+{
+    Tk_TextLayout textLayout;
+
+    /*
+     * The justification itself doesn't affect the geometry (size) of 
+     * the text string. We pass TK_JUSTIFY_LEFT.
+     */
+
+    textLayout = Tk_ComputeTextLayout(font,
+	string, numChars, wrapLength, TK_JUSTIFY_LEFT, 0,
+	widthPtr, heightPtr);
+    Tk_FreeTextLayout(textLayout);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TixDisplayText --
+ *
+ *	Display a text string on one or more lines.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The text given by "string" gets displayed at the given location
+ *	in the given drawable with the given font etc.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TixDisplayText(display, drawable, font, string, numChars, x, y,
+	length, justify, underline, gc)
+    Display *display;		/* X display to use for drawing text. */
+    Drawable drawable;		/* Window or pixmap in which to draw the
+				 * text. */
+    TixFont font;		/* Font that determines geometry of text
+				 * (should be same as font in gc). */
+    char *string;		/* String to display;  may contain embedded
+				 * newlines. */
+    int numChars;		/* Number of characters to use from string. */
+    int x, y;			/* Pixel coordinates within drawable of
+				 * upper left corner of display area. */
+    int length;			/* Line length in pixels;  used to compute
+				 * word wrap points and also for
+				 * justification.   Must be > 0. */
+    Tk_Justify justify;		/* How to justify lines. */
+    int underline;		/* Index of character to underline, or < 0
+				 * for no underlining. */
+    GC gc;			/* Graphics context to use for drawing text. */
+{
+    Tk_TextLayout textLayout;
+    int dummy;
+
+    textLayout = Tk_ComputeTextLayout(font,
+	string, numChars, length, justify, 0,
+	&dummy, &dummy);
+
+    Tk_DrawTextLayout(display, drawable, gc, textLayout,
+	    x, y, 0, -1);
+    Tk_UnderlineTextLayout(display, drawable, gc,
+	    textLayout, x, y, underline);
+
+    Tk_FreeTextLayout(textLayout);
+}
+#endif
