@@ -11,10 +11,8 @@
  */
 
 #include <tclInt.h>
-#include <tkPort.h>
-#include <tk.h>
-#include <tix.h>
-
+#include <tixInt.h>
+#include <tixItcl.h>
 
 static int			ParseOptions _ANSI_ARGS_((
 				    Tcl_Interp * interp,TixClassRecord * cPtr,
@@ -28,7 +26,8 @@ TIX_DECLARE_CMD(Tix_InstanceCmd);
  * 	Create an instance object of a Tix widget class.
  *
  * argv[0]  = object name.
- * argv[1+] = args 
+ * argv[1+] = args
+ *----------------------------------------------------------------------
  */
 TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 {
@@ -42,11 +41,7 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
     int code = TCL_OK;
     Tk_Window mainWin = Tk_MainWindow(interp);
 
-#ifdef ITCL_20
-    register Interp *iPtr = (Interp *) interp;
-    CallFrame *savedVarFramePtr;
-    Itcl_ActiveNamespace nsToken = NULL;
-#endif
+    DECLARE_ITCL_NAMESP(nameSp, interp);
 
     if (argc <= 1) {
 	return Tix_ArgcError(interp, argc, argv, 1, "pathname ?arg? ...");
@@ -63,16 +58,10 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 	Tcl_ResetResult(interp);
     }
 
-#ifdef ITCL_20
-    savedVarFramePtr = iPtr->varFramePtr;
-    iPtr->varFramePtr = NULL;
-
-    nsToken = Itcl_ActivateNamesp(interp, (Itcl_Namespace)iPtr->globalNs);
-    if (nsToken == NULL) {
+    if (!TixItclSetGlobalNameSp(&nameSp, interp)) {
         code = TCL_ERROR;
 	goto done;
     }
-#endif
 
     /*
      * Before doing anything, let's reset the TCL result, errorInfo,
@@ -122,7 +111,7 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
     }
 
     Tcl_CreateCommand(interp, widRec, Tix_InstanceCmd,
-	(ClientData)cPtr, (void (*)()) NULL);
+	(ClientData)cPtr, NULL);
 
     /* Now call the initialization methods defined by the Tix Intrinsics
      */
@@ -176,6 +165,9 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 	 */
 	if (interp->result) {
 	    oldResult = (char*)strdup(interp->result);
+#if 0
+	    printf("%s -->\n%s\n", widRec, oldResult);
+#endif
 	} else {
 	    oldResult = NULL;
 	}
@@ -188,7 +180,11 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 	topLevel = cPtr->mainWindow;
 
 	if (widRec != NULL) {
-	    if (tkwin = Tk_NameToWindow(interp, widRec, topLevel)) {
+	    Display *display = NULL;
+
+	    tkwin = Tk_NameToWindow(interp, widRec, topLevel);
+	    if (tkwin != NULL) {
+		display = Tk_Display(tkwin);
 		Tk_DestroyWindow(tkwin);
 	    }
 
@@ -198,6 +194,17 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 
 	    /* (3) widget record */
 	    Tcl_UnsetVar(interp, widRec, TCL_GLOBAL_ONLY);
+
+	    if (display) {
+#ifndef _WINDOWS
+		XSync(display, False);
+#endif
+		while (1) {
+		    if (Tk_DoOneEvent(TK_X_EVENTS|TK_DONT_WAIT) == 0) {
+			break;
+		    }
+		}
+	    }
 	}
 	if (oldResult) {
 	    Tcl_SetResult(interp, oldResult, TCL_DYNAMIC);
@@ -219,13 +226,7 @@ TIX_DEFINE_CMD(Tix_CreateWidgetCmd)
 	ckfree(rootCmd);
     }
 
-#ifdef ITCL_20
-    if (nsToken != NULL) {
-	Itcl_DeactivateNamesp(interp, nsToken);
-    }
-    iPtr->varFramePtr = savedVarFramePtr;
-#endif
-
+    TixItclRestoreGlobalNameSp(&nameSp, interp);
 
     return code;
 }
@@ -244,11 +245,9 @@ static int ParseOptions(interp, cPtr, widRec, argc, argv)
     char** argv;
 {
     int i;
-    int flag = TCL_GLOBAL_ONLY;
     TixConfigSpec *spec;
     Tk_Window tkwin;
     char * value;
-    static Tk_Window topLevel = NULL;
 
     if ((argc %2) != 0) {
 	Tcl_AppendResult(interp, "missing argument for \"", argv[argc-1],
@@ -256,11 +255,7 @@ static int ParseOptions(interp, cPtr, widRec, argc, argv)
 	return TCL_ERROR;
     }
 
-    if (topLevel == NULL) {
-	topLevel = Tk_MainWindow(interp);
-    }
-
-    if ((tkwin = Tk_NameToWindow(interp, widRec, topLevel)) == NULL) {
+    if ((tkwin = Tk_NameToWindow(interp, widRec, cPtr->mainWindow)) == NULL) {
 	return TCL_ERROR;
     }
 
