@@ -27,22 +27,22 @@ tixWidgetClass tixControl {
     -configspec {
 	{-allowempty allowEmpty AllowEmpty false}
 	{-autorepeat autoRepeat AutoRepeat true}
-	{-command command Command {}}
-	{-decrcmd decrCmd DecrCmd {}}
+	{-command command Command ""}
+	{-decrcmd decrCmd DecrCmd ""}
 	{-disablecallback disableCallback DisableCallback 0 tixVerifyBoolean}
 	{-disabledforeground disabledForeground DisabledForeground #303030}
-	{-incrcmd incrCmd IncrCmd {}}
+	{-incrcmd incrCmd IncrCmd ""}
 	{-initwait initWait InitWait 500}
 	{-integer integer Integer false}
-	{-max max Max {}}
-	{-min min Min {}}
+	{-max max Max ""}
+	{-min min Min ""}
 	{-repeatrate repeatRate RepeatRate 50}
 	{-step step Step 1}
 	{-state state State normal}
 	{-selectmode selectMode SelectMode normal}
-	{-validatecmd validateCmd ValidateCmd {}}
+	{-validatecmd validateCmd ValidateCmd ""}
 	{-value value Value 0}
-	{-variable variable Variable {}}
+	{-variable variable Variable ""}
     }
     -alias {
 	{-llimit -min}
@@ -63,7 +63,7 @@ tixWidgetClass tixControl {
     }
 }
 
-proc tixControl::InitWidgetRec {w} {
+proc tixControl:InitWidgetRec {w} {
     upvar #0 $w data
 
     tixChainMethod $w InitWidgetRec
@@ -72,7 +72,7 @@ proc tixControl::InitWidgetRec {w} {
     set data(serial)	0
 }
 
-proc tixControl::ConstructFramedWidget {w frame} {
+proc tixControl:ConstructFramedWidget {w frame} {
     upvar #0 $w data
 
     tixChainMethod $w ConstructFramedWidget $frame
@@ -100,18 +100,18 @@ proc tixControl::ConstructFramedWidget {w frame} {
     set data(labelfg) [$data(w:label) cget -fg]
 }
 
-proc tixControl::SetBindings {w} {
+proc tixControl:SetBindings {w} {
     upvar #0 $w data
 
     tixChainMethod $w SetBindings
 
-    bind $data(w:incr) <ButtonPress-1> "tixControl::StartRepeat $w  1"
-    bind $data(w:decr) <ButtonPress-1> "tixControl::StartRepeat $w -1"
+    bind $data(w:incr) <ButtonPress-1> "tixControl:StartRepeat $w  1"
+    bind $data(w:decr) <ButtonPress-1> "tixControl:StartRepeat $w -1"
 
     # These bindings will stop the button autorepeat when the 
     # mouse button is up
     foreach btn "$data(w:incr) $data(w:decr)" {
-	bind $btn <ButtonRelease-1> "tixControl::StopRepeat $w"
+	bind $btn <ButtonRelease-1> "tixControl:StopRepeat $w"
     }
 
     tixSetMegaWidget $data(w:entry) $w
@@ -121,35 +121,37 @@ proc tixControl::SetBindings {w} {
     tixAddBindTag $data(w:entry) TixControl:Entry 
 }
 
-bind TixControl:Entry <Return> {
-    tixControl::invoke [tixGetMegaWidget %W]
-}
-bind TixControl:Entry <Escape> {
-    tixControl::Escape [tixGetMegaWidget %W]
-}
-bind TixControl:Entry <Up> {
-    [tixGetMegaWidget %W] incr
-}
-bind TixControl:Entry <Down> {
-    [tixGetMegaWidget %W] decr
-}
-bind TixControl:Entry <FocusOut> {
-    if {"%d" == "NotifyNonlinear" || "%d" == "NotifyNonlinearVirtual"} {
-	tixControl::Tab [tixGetMegaWidget %W] %d
+proc tixControlBind {} {
+    tixBind TixControl:Entry <Return> {
+	tixControl:Invoke [tixGetMegaWidget %W] 1
+    }
+    tixBind TixControl:Entry <Escape> {
+	tixControl:Escape [tixGetMegaWidget %W]
+    }
+    tixBind TixControl:Entry <Up> {
+	[tixGetMegaWidget %W] incr
+    }
+    tixBind TixControl:Entry <Down> {
+	[tixGetMegaWidget %W] decr
+    }
+    tixBind TixControl:Entry <FocusOut> {
+	if {"%d" == "NotifyNonlinear" || "%d" == "NotifyNonlinearVirtual"} {
+	    tixControl:Tab [tixGetMegaWidget %W] %d
+	}
+    }
+    tixBind TixControl:Entry <Any-KeyPress> {
+	tixControl:KeyPress [tixGetMegaWidget %W]
+    }
+    tixBind TixControl:Entry <Any-Tab> {
+	# This has a higher priority than the <Any-KeyPress>  binding
+	# --> so that data(edited) is not set
     }
-}
-bind TixControl:Entry <Any-KeyPress> {
-    tixControl::KeyPress [tixGetMegaWidget %W]
-}
-bind TixControl:Entry <Any-Tab> {
-    # This has a higher priority than the <Any-KeyPress>  binding
-    # --> so that data(edited) is not set
 }
 
 #----------------------------------------------------------------------
 #                           CONFIG OPTIONS
 #----------------------------------------------------------------------
-proc tixControl::config-state {w arg} {
+proc tixControl:config-state {w arg} {
     upvar #0 $w data
 
     if {$arg == "normal"} {
@@ -169,10 +171,10 @@ proc tixControl::config-state {w arg} {
     }
 }
 
-proc tixControl::config-value {w value} {
+proc tixControl:config-value {w value} {
     upvar #0 $w data
 
-    tixControl::SetValue $w $value 0 1
+    tixControl:SetValue $w $value 0 1
 
     # This will tell the Intrinsics: "Please use this value"
     # because "value" might be altered by SetValues
@@ -180,13 +182,13 @@ proc tixControl::config-value {w value} {
     return $data(-value)
 }
 
-proc tixControl::config-variable {w arg} {
+proc tixControl:config-variable {w arg} {
     upvar #0 $w data
 
     if [tixVariable:ConfigVariable $w $arg] {
        # The value of data(-value) is changed if tixVariable:ConfigVariable 
        # returns true
-       tixControl::SetValue $w $data(-value) 1 1
+       tixControl:SetValue $w $data(-value) 1 1
     }
     catch {
 	unset data(varInited)
@@ -197,7 +199,7 @@ proc tixControl::config-variable {w arg} {
 #----------------------------------------------------------------------
 #                         User Commands
 #----------------------------------------------------------------------
-proc tixControl::incr {w {by 1}} {
+proc tixControl:incr {w {by 1}} {
     upvar #0 $w data
 
     if {$data(-state) != "disabled"} {
@@ -206,12 +208,12 @@ proc tixControl::incr {w {by 1}} {
 	    $data(w:entry) select to   end
 	}
 	
-	tixControl::SetValue $w [$data(w:entry) get] 0 1
-	tixControl::AdjustValue $w $by
+	tixControl:SetValue $w [$data(w:entry) get] 0 1
+	tixControl:AdjustValue $w $by
     }
 }
 
-proc tixControl::decr {w {by 1}} {
+proc tixControl:decr {w {by 1}} {
     upvar #0 $w data
 
     if {$data(-state) != "disabled"} {
@@ -220,12 +222,133 @@ proc tixControl::decr {w {by 1}} {
 	    $data(w:entry) select to   end
 	}
 
-	tixControl::SetValue $w [$data(w:entry) get] 0 1
-	tixControl::AdjustValue $w [expr 0 - $by]
+	tixControl:SetValue $w [$data(w:entry) get] 0 1
+	tixControl:AdjustValue $w [expr 0 - $by]
     }
 }
 
-proc tixControl::invoke {w} {
+proc tixControl:invoke {w} {
+    upvar #0 $w data
+
+    tixControl:Invoke $w 0
+}
+
+proc tixControl:update {w} {
+    upvar #0 $w data
+
+    if [info exists data(edited)] {
+	tixControl:invoke $w
+    }
+}
+
+#----------------------------------------------------------------------
+#                       Internal Commands
+#----------------------------------------------------------------------
+
+# Change the value by a multiple of the data(-step)
+#
+proc tixControl:AdjustValue {w amount} {
+    upvar #0 $w data
+
+    if {$amount == 1 && $data(-incrcmd) != ""} {
+	set newValue [tixEvalCmdBinding $w $data(-incrcmd) "" $data(-value)]
+    } elseif {$amount == -1 && $data(-decrcmd) != ""} {
+	set newValue [tixEvalCmdBinding $w $data(-decrcmd) "" $data(-value)]
+    } else {
+	set newValue [expr $data(-value) + $amount * $data(-step)]
+    }
+
+    if {$data(-state) != "disabled"} {
+	tixControl:SetValue $w $newValue 0 1
+    }
+}
+
+proc tixControl:SetValue {w newvalue noUpdate forced} {
+    upvar #0 $w data
+
+    set oldvalue $data(-value)
+    set oldCursor [$data(w:entry) index insert]
+    set changed 0
+
+
+    if {$data(-validatecmd) != ""} {
+	# Call the user supplied validation command
+	#
+       set data(-value) [tixEvalCmdBinding $w $data(-validatecmd) "" $newvalue]
+    } else {
+	# Here we only allow int or floating numbers
+	#
+	# If the new value is not a valid number, the old value will be
+	# kept due to the "catch" statements
+	#
+	if [catch {expr 0+$newvalue}] {
+	    set newvalue 0
+	    set data(-value) 0
+	    set changed 1
+	}
+
+	if {$newvalue == ""} {
+	    if {![tixGetBoolean -nocomplain $data(-allowempty)]} {
+		set newvalue 0
+		set changed 1
+	    } else {
+		set data(-value) ""
+	    }
+	}
+
+	if {$newvalue != ""} {
+	    # Change this to a valid decimal string (trim leading 0)
+	    #
+	    regsub {^[0]*} $newvalue "" newvalue
+	    if [catch {expr 0+$newvalue}] {
+		set newvalue 0
+		set data(-value) 0
+		set changed 1
+	    }
+	    if {$newvalue == ""} {
+		set newvalue 0
+	    }
+
+	    if [tixGetBoolean -nocomplain $data(-integer)] {
+		set data(-value) [tixGetInt -nocomplain $newvalue]
+	    } else {
+		if [catch {set data(-value) [format "%d" $newvalue]}] {
+		    if [catch {set data(-value) [expr $newvalue+0.0]}] {
+			set data(-value) $oldvalue
+		    }
+		}
+	    }
+	    
+	    # Now perform boundary checking
+	    #
+	    if {$data(-max) != "" && $data(-value) > $data(-max)} {
+		set data(-value) $data(-max)
+	    }
+	    if {$data(-min) != "" && $data(-value) < $data(-min)} {
+		set data(-value) $data(-min)
+	    }
+	}
+    }
+
+    if {! $noUpdate} {
+	tixVariable:UpdateVariable $w
+    }
+
+    if {$forced || "x$newvalue" != "x$data(-value)" || $changed} {
+	$data(w:entry) delete 0 end
+	$data(w:entry) insert 0 $data(-value)
+	$data(w:entry) icursor $oldCursor
+    }
+
+    if {!$data(-disablecallback) && $data(-command) != ""} {
+	if {![info exists data(varInited)]} {
+	    set bind(specs) ""
+	    tixEvalCmdBinding $w $data(-command) bind $data(-value)
+	}
+    }
+}
+
+proc tixControl:Invoke {w forced} {
     upvar #0 $w data
 
     catch {
@@ -239,122 +362,7 @@ proc tixControl::invoke {w} {
 	$data(w:entry) select to   end
     }
 
-    tixControl::SetValue $w [$data(w:entry) get] 0 0
-}
-
-proc tixControl::update {w} {
-    upvar #0 $w data
-
-    if [info exists data(edited)] {
-	tixControl::invoke $w
-    }
-}
-
-#----------------------------------------------------------------------
-#                       Internal Commands
-#----------------------------------------------------------------------
-
-# Change the value by a multiple of the data(-step)
-#
-proc tixControl::AdjustValue {w amount} {
-    upvar #0 $w data
-
-    if {$amount == 1 && $data(-incrcmd) != {}} {
-	set newValue [tixEvalCmdBinding $w $data(-incrcmd) {} $data(-value)]
-    } elseif {$amount == -1 && $data(-decrcmd) != {}} {
-	set newValue [tixEvalCmdBinding $w $data(-decrcmd) {} $data(-value)]
-    } else {
-	set newValue [expr $data(-value) + $amount * $data(-step)]
-    }
-
-    if {$data(-state) != "disabled"} {
-	tixControl::SetValue $w $newValue 0 1
-    }
-}
-
-proc tixControl::SetValue {w newvalue noUpdate forced} {
-    upvar #0 $w data
-
-    set oldvalue $data(-value)
-    set oldCursor [$data(w:entry) index insert]
-    set changed 0
-
-
-    if {$data(-validatecmd) != {}} {
-	# Call the user supplied validation command
-	#
-       set data(-value) [tixEvalCmdBinding $w $data(-validatecmd) {} $newvalue]
-    } else {
-	# Here we only allow int or floating numbers
-	#
-	# If the new value is not a valid number, the old value will be
-	# kept due to the "catch" statements
-	#
-	if [catch {expr 0+$newvalue}] {
-	    set newvalue 0
-	    set data(-value) 0
-	    set changed 1
-	}
-
-	if {$newvalue == {}} {
-	    if {![tixGetBoolean -nocomplain $data(-allowempty)]} {
-		set newvalue 0
-		set changed 1
-	    } else {
-		set data(-value) {}
-	    }
-	}
-
-	if {$newvalue != {}} {
-	    # Change this to a valid decimal string (trim leading 0)
-	    #
-	    regsub {^[0]*} $newvalue {} newvalue
-	    if [catch {expr 0+$newvalue}] {
-		set newvalue 0
-		set data(-value) 0
-		set changed 1
-	    }
-	    if {$newvalue == {}} {
-		set newvalue 0
-	    }
-
-	    if [tixGetBoolean -nocomplain $data(-integer)] {
-		set data(-value) [tixGetInt -nocomplain $newvalue]
-	    } else {
-		if [catch {set data(-value) [format "%d" $newvalue]}] {
-		    if [catch {set data(-value) [format "%f" $newvalue]}] {
-			set data(-value) $oldvalue
-		    }
-		}
-	    }
-	    
-	    # Now perform boundary checking
-	    #
-	    if {$data(-max) != {} && $data(-value) > $data(-max)} {
-		set data(-value) $data(-max)
-	    }
-	    if {$data(-min) != {} && $data(-value) < $data(-min)} {
-		set data(-value) $data(-min)
-	    }
-	}
-    }
-
-    if {! $noUpdate} {
-	tixVariable:UpdateVariable $w
-    }
-
-    if {$forced || $newvalue != $data(-value) || $changed} {
-	$data(w:entry) delete 0 end
-	$data(w:entry) insert 0 $data(-value)
-	$data(w:entry) icursor $oldCursor
-    }
-
-    if {!$data(-disablecallback) && $data(-command) != {}} {
-	if {![info exists data(varInited)]} {
-	    set bind(specs) ""
-	    tixEvalCmdBinding $w $data(-command) bind $data(-value)
-	}
-    }
+    tixControl:SetValue $w [$data(w:entry) get] 0 $forced
 }
 
 #----------------------------------------------------------------------
@@ -363,7 +371,7 @@ proc tixControl::SetValue {w newvalue noUpdate forced} {
 # repeatedly but is not hold down, the serial counter will increase
 # successively and all "after" time event handlers will be discarded
 #----------------------------------------------------------------------
-proc tixControl::StartRepeat {w amount} {
+proc tixControl:StartRepeat {w amount} {
     if {![winfo exists $w]} {
 	return
     }
@@ -377,42 +385,42 @@ proc tixControl::StartRepeat {w amount} {
 
     if [info exists data(edited)] {
 	unset data(edited)
-	tixControl::SetValue $w [$data(w:entry) get] 0 1
+	tixControl:SetValue $w [$data(w:entry) get] 0 1
     }
 
     incr data(serial)
 
-    tixControl::AdjustValue $w $amount
+    tixControl:AdjustValue $w $amount
 
     if {$data(-autorepeat)} {
-	after $data(-initwait) tixControl::Repeat $w $amount $data(serial)
+	after $data(-initwait) tixControl:Repeat $w $amount $data(serial)
     }
 
     focus $data(w:entry)
 }
 
-proc tixControl::Repeat {w amount serial} {
+proc tixControl:Repeat {w amount serial} {
     if {![winfo exists $w]} {
 	return
     }
     upvar #0 $w data
 
     if {$serial == $data(serial)} {
-	tixControl::AdjustValue $w $amount
+	tixControl:AdjustValue $w $amount
 
 	if {$data(-autorepeat)} {
-	   after $data(-repeatrate) tixControl::Repeat $w $amount $data(serial)
+	   after $data(-repeatrate) tixControl:Repeat $w $amount $data(serial)
 	}
     }
 }
 
-proc tixControl::StopRepeat {w} {
+proc tixControl:StopRepeat {w} {
     upvar #0 $w data
 
     incr data(serial)
 }
 
-proc tixControl::Destructor {w} {
+proc tixControl:Destructor {w} {
 
     tixVariable:DeleteVariable $w
 
@@ -423,7 +431,7 @@ proc tixControl::Destructor {w} {
 
 # ToDo: maybe should return -code break if the value is not good ...
 #
-proc tixControl::Tab {w detail} {
+proc tixControl:Tab {w detail} {
     upvar #0 $w data
 
     if {![info exists data(edited)]} {
@@ -432,17 +440,17 @@ proc tixControl::Tab {w detail} {
 	unset data(edited)
     }
 
-    tixControl::invoke $w
+    tixControl:invoke $w
 }
 
-proc tixControl::Escape {w} {
+proc tixControl:Escape {w} {
     upvar #0 $w data
 
     $data(w:entry) delete 0 end
     $data(w:entry) insert 0 $data(-value)
 }
 
-proc tixControl::KeyPress {w} {
+proc tixControl:KeyPress {w} {
     upvar #0 $w data
 
     if {$data(-selectmode) == "normal"} {
@@ -450,6 +458,6 @@ proc tixControl::KeyPress {w} {
 	return
     } else {
 	# == "immediate"
-	after 1 tixControl::invoke $w
+	after 1 tixControl:invoke $w
     }
 }
