@@ -1,6 +1,3 @@
-#
-#	$Id: minterp.tcl,v 1.1.1.1 2000/05/17 11:08:51 idiscovery Exp $
-#
 # minterp.tcl
 #
 #	Tests Tix running under multiple interpreters.
@@ -16,7 +13,7 @@ proc About {} {
 }
 
 proc Test {} {
-    global tix tcl_version
+    global tix
     if ![string comp [info commands interp] ""] {
 	# Does not support multiple interpreters.
 	return
@@ -26,9 +23,8 @@ proc Test {} {
 	#
 	# multiple interpreters currently core dumps under itcl2.1
 	#
-#	return
+	return
     }
-
     TestBlock minterp-1.1 {multiple interpreters} {
 	for {set x 0} {$x < 5} {incr x} {
 	    global testConfig
@@ -36,12 +32,9 @@ proc Test {} {
 	    interp eval a "set dynlib [list $testConfig(dynlib)]"
 	    if {[info exists tix(et)] && $tix(et) == 1} {
 		interp eval a {
-		    catch {load "" Tk}
-		    catch {load "" ITcl}
-		    catch {load "" ITk}
-		    catch {load "" Tclsam}
-		    catch {load "" Tksam}
-		    catch {load "" Tixsam}
+		    load "" Tclsam
+		    load "" Tksam
+		    load "" Tixsam
 		}
 	    } else {
 		interp eval a {
