@@ -1,7 +1,3 @@
-# -*-mode: tcl; fill-column: 75; tab-width: 8; coding: iso-latin-1-unix -*-
-#
-#	$Id: doxx.tcl,v 1.1.1.1.2.2 2002/12/15 04:21:53 idiscovery Exp $
-#
 # doxx.tcl --
 #
 # Common code shared by doconfig.tcl and domakefile.tcl
@@ -134,26 +130,10 @@ proc ptcl {string} {
     }
 }
 
-set ENABLE_SAM 0
-set SAM_LIB    0
-set SAM_EXE    0
-
-if {!$ITCL} {
-    if {$TCL_VERSION <= 7.6} {
-	set ENABLE_SAM 1
-	set SAM_LIB    1
-	set SAM_EXE    1
-    } else {
-	set ENABLE_SAM 1
-	set SAM_LIB    1
-	set SAM_EXE    0
-    }
+if {!$ITCL && $TCL_VERSION <= 7.6} {
+    set ENABLE_SAM 1
 } else {
-    if {$TCL_VERSION == 7.6} {
-	set ENABLE_SAM 1
-	set SAM_LIB    1
-	set SAM_EXE    0
-    }
+    set ENABLE_SAM 0
 }
 
 proc p_sam {string} {
