@@ -1,6 +1,3 @@
-
-/*	$Id: tixUnixPort.h,v 1.2 2001/01/15 10:35:57 ioilam Exp $	*/
-
 /*
  * tixUnixPort.h --
  *
@@ -21,14 +18,6 @@
 struct _TixpSubRegion {
     XRectangle rect;
     int rectUsed;
-    int origX;
-    int origY;
 };
 
-#ifdef UCHAR_SUPPORTED
-typedef unsigned char UNSIGNED_CHAR;
-#else
-typedef char UNSIGNED_CHAR;
-#endif
-
 #endif /* _TIX_UNIXPORT_H_ */
