@@ -1,6 +1,3 @@
-
-/*	$Id: tixUnixXpm.c,v 1.2 2004/03/28 02:44:57 hobbs Exp $	*/
-
 /*
  * tixUnixImgXpm.c --
  *
@@ -17,7 +14,6 @@
 #include <tixPort.h>
 #include <tixUnixInt.h>
 #include <tixImgXpm.h>
-#include <X11/Xutil.h>
 
 typedef struct PixmapData {
     Pixmap mask;		/* Mask: only display pixmap pixels where
@@ -84,20 +80,20 @@ TixpXpmAllocTmpBuffer(masterPtr, instancePtr, imagePtr, maskPtr)
     /*
      * Create the XImage structures to store the temporary image
      */
-    image = XCreateImage(display, Tk_Visual(instancePtr->tkwin),
-	    (unsigned) depth, ZPixmap, 0, 0,
-	    (unsigned) masterPtr->size[0], (unsigned) masterPtr->size[1],
-	    pad, 0);
+    image = XCreateImage(display,
+	Tk_Visual(instancePtr->tkwin),
+	depth, ZPixmap, 0, 0,
+	masterPtr->size[0], masterPtr->size[1], pad, 0);
     image->data =
-      (char *)ckalloc((unsigned) image->bytes_per_line * masterPtr->size[1]);
+      (char *)ckalloc(image->bytes_per_line * masterPtr->size[1]);
 
-    mask  = XCreateImage(display, Tk_Visual(instancePtr->tkwin),
-	    1, XYPixmap, 0, 0,
-	    (unsigned) masterPtr->size[0], (unsigned) masterPtr->size[1],
-	    pad, 0);
+    mask  = XCreateImage(display,
+	Tk_Visual(instancePtr->tkwin),
+	1, XYPixmap, 0, 0,
+	masterPtr->size[0], masterPtr->size[1], pad, 0);
 
     mask->data =
-	(char *)ckalloc((unsigned) mask->bytes_per_line  * masterPtr->size[1]);
+      (char *)ckalloc(mask->bytes_per_line  * masterPtr->size[1]);
 
     *imagePtr = image;
     *maskPtr = mask;
@@ -178,8 +174,7 @@ TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp)
     gc = Tk_GetGC(instancePtr->tkwin, 0, NULL);
 
     XPutImage(display, instancePtr->pixmap,
-	    gc, image, 0, 0, 0, 0,
-	    (unsigned) masterPtr->size[0], (unsigned) masterPtr->size[1]);
+	gc, image, 0, 0, 0, 0, masterPtr->size[0], masterPtr->size[1]);
 
     Tk_FreeGC(display, gc);
 
@@ -192,8 +187,7 @@ TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp)
 	    masterPtr->size[0], masterPtr->size[1], 1);
 	gc = XCreateGC(display, dataPtr->mask, 0, NULL);
 	XPutImage(display, dataPtr->mask,
-		gc, mask,  0, 0, 0, 0,
-		(unsigned) masterPtr->size[0], (unsigned) masterPtr->size[1]);
+	    gc, mask,  0, 0, 0, 0, masterPtr->size[0], masterPtr->size[1]);
 	XFreeGC(display, gc);
     } else {
 	dataPtr->mask = None;
@@ -216,20 +210,19 @@ TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp)
 }
 
 void
-TixpXpmFreeInstanceData(instancePtr, delete, display)
-    PixmapInstance *instancePtr;/* Pixmap instance. */
-    int delete;			/* Should the instance data structure
-				 * be deleted as well? */
-    Display *display;		/* Display containing window that used image.*/
+TixpXpmFreeInstanceData(instancePtr, delete)
+    PixmapInstance *instancePtr;	/* Pixmap instance. */
+    int delete;				/* Should the instance data structure
+					 * be deleted as well? */
 {
     PixmapData *dataPtr = (PixmapData*)instancePtr->clientData;
 
     if (dataPtr->mask != None) {
-	Tk_FreePixmap(display, dataPtr->mask);
+	Tk_FreePixmap(Tk_Display(instancePtr->tkwin), dataPtr->mask);
 	dataPtr->mask = None;
     }
     if (dataPtr->gc != None) {
-	Tk_FreeGC(display, dataPtr->gc);
+	Tk_FreeGC(Tk_Display(instancePtr->tkwin), dataPtr->gc);
 	dataPtr->gc = None;
     }
     if (delete) {
