@@ -1,5 +1,3 @@
-/*	$Id: tixWinMain.c,v 1.2.2.1 2001/11/04 05:23:02 idiscovery Exp $	*/
-
 /* 
  * tixWinMain.c --
  *
@@ -19,22 +17,12 @@
 #include <windows.h>
 #undef WIN32_LEAN_AND_MEAN
 #include <malloc.h>
-#include <locale.h>
 
 #ifdef ITCL_2
 #include "itcl.h"
 #include "itk.h"
 #endif
 
-/*
- * The following declarations refer to internal Tk routines.  These
- * interfaces are available for use, but are not supported.
- */
-void	TkConsoleCreate _ANSI_ARGS_((void));
-int	TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
-void	TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
-			    int devId, char *buffer, long size));
-
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -172,28 +160,12 @@ int
 Tcl_AppInit(interp)
     Tcl_Interp *interp;		/* Interpreter for application. */
 {
-    /*
-     * Set up the default locale to be standard "C" locale so parsing
-     * is performed correctly.
-     */
-    setlocale(LC_ALL, "C");
-
-    /*
-     * Increase the application queue size from default value of 8.
-     * At the default value, cross application SendMessage of WM_KILLFOCUS
-     * will fail because the handler will not be able to do a PostMessage!
-     * This is only needed for Windows 3.x, since NT dynamically expands
-     * the queue.
-     */
-    SetMessageQueue(64);
-
     /*
      * Create the console channels and install them as the standard
      * channels.  All I/O will be discarded until TkConsoleInit is
      * called to attach the console to a text widget.
      */
-
-    TkConsoleCreate();
+    TixConsoleCreate(interp);
 
     if (Tcl_Init(interp) == TCL_ERROR) {
 	if (Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY)) {
@@ -242,22 +214,6 @@ Tcl_AppInit(interp)
 	return TCL_ERROR;
     }
     Tcl_StaticPackage(interp, "Itk", Itk_Init, (Tcl_PackageInitProc *) NULL);
-
-
-    /*
-     *  This is itclsh, so import all [incr Tcl] commands by
-     *  default into the global namespace.  Fix up the autoloader
-     *  to do the same.
-     */
-    if (Tcl_Import(interp, Tcl_GetGlobalNamespace(interp),
-            "::itcl::*", /* allowOverwrite */ 1) != TCL_OK) {
-        return TCL_ERROR;
-    }
-
-    if (Tcl_Eval(interp, "auto_mkindex_parser::slavehook { _%@namespace import -force ::itcl::* }") != TCL_OK) {
-        return TCL_ERROR;
-    }
-
 #endif
 
     if (Tix_Init(interp) == TCL_ERROR) {
@@ -279,13 +235,12 @@ Tcl_AppInit(interp)
 
     if (strcmp(Tcl_GetVar(interp, "tcl_interactive", TCL_GLOBAL_ONLY), "1")
 	    == 0) {
-	if (TkConsoleInit(interp) == TCL_ERROR) {
+	if (TixConsoleInit(interp) == TCL_ERROR) {
 	    return TCL_ERROR;
 	}
     }
 
-    /* Now done in Tix.tcl */
-    /* Tcl_SetVar(interp, "tcl_rcFileName", "~/wishrc.tcl", TCL_GLOBAL_ONLY); */
+    Tcl_SetVar(interp, "tcl_rcFileName", "~/wishrc.tcl", TCL_GLOBAL_ONLY);
     return TCL_OK;
 }
 
