@@ -1,6 +1,3 @@
-
-/*	$Id: tixWinXpm.c,v 1.1.1.1.2.1 2002/12/11 04:51:51 idiscovery Exp $	*/
-
 /*
  * tixWinImgXpm.c --
  *
@@ -74,11 +71,6 @@ TixpXpmAllocTmpBuffer(masterPtr, instancePtr, imagePtr, maskPtr)
     XImage * mask;
     Display *display = Tk_Display(instancePtr->tkwin);
     int depth;
-    int maskSize;
-    int i;
-    int wordBits  = sizeof(WORD)*8;		/* # of bits in WORD */
-    int wordBytes = sizeof(WORD)/sizeof(char);	/* # of bytes in WORD */
-    int words_per_line;
 
     depth = Tk_Depth(instancePtr->tkwin);
 
@@ -90,19 +82,10 @@ TixpXpmAllocTmpBuffer(masterPtr, instancePtr, imagePtr, maskPtr)
 
     mask->width = masterPtr->size[0];
     mask->height = masterPtr->size[1];
+    mask->bytes_per_line = (mask->width+7)/8;
 
-    /*
-     * In Windows, each scan line in the the mask data must be aligned
-     * to words. The padding bits must be zero'ed.
-     */
-    words_per_line = (mask->width + (wordBits-1))/wordBits;
-    mask->bytes_per_line = words_per_line * wordBytes;
-
-    maskSize = mask->bytes_per_line  * mask->height;
-    mask->data = (char *)ckalloc(maskSize);
-    for (i=0; i<maskSize; i++) {
-	mask->data[i] = 0;
-    }
+    mask->data =
+      (char *)ckalloc(mask->bytes_per_line  * masterPtr->size[1]);
 
     *imagePtr = image;
     *maskPtr = mask;
@@ -218,6 +201,7 @@ TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp)
 	BitBlt(bitmapDC, 0, 0, w, h, maskDC, 0, 0, SRCAND);
 	BitBlt(maskDC,   0, 0, w, h, maskDC, 0, 0, NOTSRCCOPY);
 
+	TkWinReleaseDrawableDC(instancePtr->pixmap, dc, &dcState);
 	dataPtr->maskDC = maskDC;
 	dataPtr->maskBm = maskBm;
 	dataPtr->maskBmOld = maskBmOld;
@@ -227,15 +211,13 @@ TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp)
     dataPtr->bitmapDC = bitmapDC;
     dataPtr->bitmap = bitmap;
     dataPtr->bitmapOld = bitmapOld;
-    TkWinReleaseDrawableDC(instancePtr->pixmap, dc, &dcState);
 }
 
 void
-TixpXpmFreeInstanceData(instancePtr, delete, display)
+TixpXpmFreeInstanceData(instancePtr, delete)
     PixmapInstance *instancePtr;	/* Pixmap instance. */
     int delete;				/* Should the instance data structure
 					 * be deleted as well? */
-    Display * display;			/* Unused on Windows. */
 {
     PixmapData *dataPtr = (PixmapData*)instancePtr->clientData;
 
